<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="../../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:gu="urn:X-gc2ublndr"
                exclude-result-prefixes="xs gu xsd"
                version="2.0">

<xs:doc info="$Id: Crane-commonndr.xsl,v 1.2 2011/11/27 19:50:18 admin Exp $"
        filename="Crane-commonndr.xsl" vocabulary="DocBook" internal-ns="gu">
  <xs:title>Common components to Crane's NDR stylesheets</xs:title>
  <para> 
    This stylesheet includes the common bits between Crane's stylesheets
    acting on genericode files of CCTS entities.
  </para>

  <section>
    <title>Copyright and disclaimer</title>
<programlisting>Copyright (C) - Crane Softwrights Ltd.
              - <ulink url="http://www.CraneSoftwrights.com/links/res-dev.htm">http://www.CraneSoftwrights.com/links/res-dev.htm</ulink>

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at <ulink
url="http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5">http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5</ulink>

THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR ANY
PURPOSE.</programlisting>
    </section>
</xs:doc>

<!--========================================================================-->
<xs:doc id="ingc">
  <xs:title>Input genericode file assumptions</xs:title>
  <para>
    The main input genericode file defines the business information entities
    of the schema fragments being generated.  When generating the common
    fragments, this is the common genericode file definition.  When generating
    an extension or additional document schema fragments, this is the
    genericode file definition for those fragments, and the common genericode
    file definition is supplied in an invocation argument.
  </para>
  <para>
    The column names of the input genericode file serialization of the UBL
    NDR spreadsheets are assumed to include at a minimum the following:
  </para>
  <itemizedlist>
    <listitem><literal>ModelName</literal> - the full model name</listitem>
    <listitem><literal>UBLName</literal> or <literal>ComponentName</literal> - the component name</listitem>
    <listitem><literal>ComponentType</literal> - ABIE, BBIE or ASBIE</listitem>
    <listitem><literal>Cardinality</literal> - 1, 0..1, 0..n or 1..n</listitem>
    <listitem><literal>ObjectClass</literal> - ISO/IEC 11179 classification part</listitem>
    <listitem><literal>AssociatedObjectClass</literal> - ISO/IEC 11179 classification part</listitem>
    <listitem><literal>DataType</literal> - CCTS V2.01 qualified data type</listitem>
  </itemizedlist>
  <para>
    The OASIS genericode specification is available at
    <ulink url="http://docs.oasis-open.org/codelist/genericode/">
      <literal>﻿http://docs.oasis-open.org/codelist/genericode/</literal>
    </ulink>.
  </para>
  <para>
    An example serialization is found in the UBL 2.1 PRD2 distribution at
    <ulink url="http://docs.oasis-open.org/ubl/prd2-UBL-2.1/mod/UBL-Entities-2.1.gc">
      <literal>http://docs.oasis-open.org/ubl/prd2-UBL-2.1/mod/UBL-Entities-2.1.gc</literal>
    </ulink>.
  </para>
</xs:doc>

<!--========================================================================-->
<xs:doc>
  <xs:title>Invocation parameters</xs:title>
</xs:doc>

<xs:param ignore-ns="yes">
  <para>
    When generating an extension or additional document schema fragments, this
    must be specified to supply the common genericode file definition, since
    the input genericode file definition is for the generating schema fragment.
  </para>
</xs:param>
<xsl:param name="common-gc-uri" as="xsd:anyURI?" select="()"/>

<xs:param name="common-config-uri" ignore-ns="yes">
  <para>
    This is the location of the common configuration file used to express
    where common components are found when creating extension or additional
    document schemas.  No schema fragments are created for the artefacts
    described in this file, only the artefacts in the 
    <literal>configuration-uri=</literal> file.
    It is assumed that the base output directory
    expressed by this configuration file is the same as that expressed by
    the main configuration file, such that common subdirectories can be
    matched.
  </para>
</xs:param>
<xsl:param name="common-config-uri" select="()" as="xsd:anyURI?"/>

<xs:param ignore-ns="yes">
  <para id="invoke">
    The filename of the main configuration file as a URI.  A relative URI
    is resolved relative to the base URI of the input genericode file.
  </para>
  <para>
    The configuration file describes all of the directories and files that are
    created or files that are needed by the files that are created.
    The file types are indicated using the <literal>type=</literal> attribute.
    The files that are created are:
  </para>
  <itemizedlist>
    <listitem><literal>CVA</literal> - created context/value association file
     <itemizedlist>
       <listitem>
         there are as many of these entries as CVA files need to be created
       </listitem>
       <listitem>
         each CVA file points to a skeleton CVA file in which
         <literal>&lt;Context></literal> elements have invalid
         <literal>NDRDataType=</literal> attributes that are each replaced
         with a valid <literal>address=</literal> attribute of the union of all
         BBIE elements, by name using the BBIE namespace prefix, that use
         the given qualified data type (e.g. 
         <literal>NDRDataType="Currency_ Code. Type"</literal>)
       </listitem>
     </itemizedlist>
    </listitem>
    <listitem><literal>SABIE</literal> - created supplemental library ABIE schema file</listitem>
    <listitem><literal>SBBIE</literal> - created supplemental library BBIE schema file</listitem>
    <listitem><literal>XABIE</literal> - created extension point ABIE schema file</listitem>
    <listitem><literal>AABIE</literal> - created additional document ABIE schema file</listitem>
    <listitem><literal>CABIE</literal> - created common library ABIE schema file</listitem>
    <listitem><literal>CBBIE</literal> - created common library BBIE schema file</listitem>
    <listitem><literal>DABIE</literal> - created document ABIE schema file
      <itemizedlist>
        <listitem>
          there are as many of these <literal>DABIE</literal> entries as
          document schemas need to be created
        </listitem>
        <listitem>
          when supplemental/extension/additional schema files are being created,
          and only one set of such schema files can be created in any given
          execution, the common schema files are not created
        </listitem>
        <listitem>
          when the element name is <literal>&lt;file></literal>, a single
          file is created with the given file name
        </listitem>
        <listitem>
          when the element name is <literal>&lt;files></literal>, all
          document ABIE files are created using the name and namespace as
          a substitution pattern where "<literal>%n</literal>" is replaced
          with the name of the document ABIE (not the model name)
        </listitem>
        <listitem>
          note that the <literal>&lt;files></literal> directive will not
          create a file for a document ABIE for which there is a corresponding
          <literal>&lt;file></literal> directive
        </listitem>
      </itemizedlist>
    </listitem>
  </itemizedlist>
  <para>The files that are referenced are:</para>
  <itemizedlist>
    <listitem><literal>QDT</literal> - referenced qualified data types file</listitem>
    <listitem><literal>UDT</literal> - referenced unqualified data types file</listitem>
    <listitem><literal>DOC</literal> - documentation namespace (no actual file)</listitem>
    <listitem><literal>EXT</literal> - referenced extension content file (optional)
      <itemizedlist>
        <listitem>
          the child of this entry is an XSD reference to the extension point
          to be included in every document schema as the first child of the
          document element
        </listitem>
      </itemizedlist>
    </listitem>
  </itemizedlist>
  <para>
    The configuration file is organized by multiple directory
    parent elements that are used to calculate relative URI references for
    XSD import directives.  There need not be any directory entries.
    The directory entry with the attribute <literal>runtime-name=</literal>
    triggers the recreation of the suite of contained files in the given
    alternative directory, but with XSD documentation constructs removed.
  </para>
  <para>
    All allowed abbreviations must be itemized as these are the portions of
    element names that do not satisfy the UpperCamelCase convention.
  </para>
  <para>
    All allowed unqualified data types must be itemized to ensure nothing
    unsupported is asked for.
  </para>
  <para>
    A common comment can be defined for all files for which a specific comment
    is not provided.  This comment is placed at the start of the file created.
    There are a number of substitution variables available for this comment:
  </para>
  <itemizedlist>
    <listitem><literal>%%</literal> - a single percent sign</listitem>
    <listitem><literal>%f</literal> - the output filename (with config path)</listitem>
    <listitem><literal>%n</literal> - the output module name (no config path)</listitem>
    <listitem><literal>%t</literal> - the local time of file creation</listitem>
    <listitem><literal>%z</literal> - the UTC (Zulu) time of file creation</listitem>
  </itemizedlist>
  <para>
    The example for a test file similar to the UBL project (but modified to 
    illustrate distinctions between
    <literal>&lt;files></literal> and <literal>&lt;file></literal> for
    documents) is as follows:
  </para>
<programlisting><![CDATA[<configuration xmlns:xsd="http://www.w3.org/2001/XMLSchema"
               version="2.1">
  <abbreviations>
    <abbreviation>CV2</abbreviation>
    <abbreviation>ID</abbreviation>
    <abbreviation>URI</abbreviation>
    <abbreviation>UNDG</abbreviation>
    <abbreviation>UBL</abbreviation>
    <abbreviation>UUID</abbreviation>
    <abbreviation>XPath</abbreviation>
  </abbreviations>
  <types>
    <type>Amount</type>
    <type>Binary Object</type>
    <type>Code</type>
    <type>Date Time</type>
    <type>Date</type>
    <type>Graphic</type>
    <type>Identifier</type>
    <type>Indicator</type>
    <type>Measure</type>
    <type>Name</type>
    <type>Numeric</type>
    <type>Percent</type>
    <type>Picture</type>
    <type>Quantity</type>
    <type>Rate</type>
    <type>Sound</type>
    <type>Text</type>
    <type>Time</type>
    <type>Value</type>
    <type>Video</type>
  </types>
  <comment>
  Library:           OASIS Universal Business Language (UBL) 2.1
                     http://docs.oasis-open.org/ubl/os-UBL-2.1/
  Module:            %f
  Generated on:      %z
</comment>
    <copyright position="end">
  Test copyright statement.
</copyright>
    <type-documentation>
      <ccts:Component xmlns:ccts="urn:un:unece:uncefact:documentation:2">
         <ccts:ComponentType>ComponentType</ccts:ComponentType>
         <ccts:DictionaryEntryName>DictionaryEntryName</ccts:DictionaryEntryName>
         <ccts:Definition>Definition</ccts:Definition>
         <ccts:Examples>Examples</ccts:Examples>
         <ccts:Cardinality>Cardinality</ccts:Cardinality>
         <ccts:ObjectClass>ObjectClass</ccts:ObjectClass>
         <ccts:PropertyTermQualifier>PropertyTermQualifier</ccts:PropertyTermQualifier>
         <ccts:PropertyTerm>PropertyTerm</ccts:PropertyTerm>
         <ccts:AssociatedObjectClass>AssociatedObjectClass</ccts:AssociatedObjectClass>
         <ccts:RepresentationTerm>RepresentationTerm</ccts:RepresentationTerm>
         <ccts:DataTypeQualifier>DataTypeQualifier</ccts:DataTypeQualifier>
         <ccts:DataType>DataType</ccts:DataType>
         <ccts:AlternativeBusinessTerms>AlternativeBusinessTerms</ccts:AlternativeBusinessTerms>
      </ccts:Component>
    </type-documentation>
    <dir name="cva">
      <file type="CVA" name="UBL-DefaultDTQ-2.1.cva"
            skeleton-uri="../ubl/UBL-2.1-CVA-Skeleton.cva"/>
    </dir>
    <dir name="xsd" runtime-name="xsdrt">
      <dir name="common">
        <file type="CABIE" name="UBL-CommonAggregateComponents-2.1.xsd"
              prefix="cac"
namespace="urn:oasis:names:specification:ubl:schema:xsd:CommonAggregateComponents-2"/>
        <file type="CBBIE" name="UBL-CommonBasicComponents-2.1.xsd"
              prefix="cbc"
namespace="urn:oasis:names:specification:ubl:schema:xsd:CommonBasicComponents-2"/>
        <file type="QDT" name="UBL-QualifiedDataTypes-2.1.xsd"
              prefix="qdt"
namespace="urn:oasis:names:specification:ubl:schema:xsd:QualifiedDataTypes-2"/>
        <file type="UDT" name="UBL-UnqualifiedDataTypes-2.1.xsd"
              prefix="udt"
namespace="urn:oasis:names:specification:ubl:schema:xsd:UnqualifiedDataTypes-2"/>
        <file type="EXT" name="UBL-CommonExtensionComponents-2.1.xsd"
              prefix="ext"
namespace="urn:oasis:names:specification:ubl:schema:xsd:CommonExtensionComponents-2">
           <xsd:element ref="ext:UBLExtensions" minOccurs="0" maxOccurs="1">
              <xsd:annotation>
                 <xsd:documentation>A container for all extensions present in the document.</xsd:documentation>
              </xsd:annotation>
           </xsd:element>
        </file>
      </dir>
      <dir name="maindoc">
        <!--this is a typical entry and most likely the only entry for docs-->
        <files type="DABIE" name="UBL-%n-2.1.xsd"
               namespace="urn:oasis:names:specification:ubl:schema:xsd:%n-2">
          <element-documentation>This element MUST be conveyed as the root
element in any instance document based on this Schema expression</element-documentation>
        </files>
        <!--this is an atypical entry, used only for exceptions to the rule-->
        <file type="DABIE" name="UBL-InvoiceShort-2.1.xsd" abie="Invoice"
              namespace="urn:oasis:names:specification:ubl:schema:xsd:InvoiceShort-2">
          <comment>
  Library:           OASIS Universal Business Language (UBL) 2.1
                     http://docs.oasis-open.org/ubl/os-UBL-2.1/
  Module (test):     %f and %% and %n
  Generated on:      %t or %z
</comment>
          <copyright position="start">
Test copyright statement.
          </copyright>
          <type-documentation>
            <cctstest:Component xmlns:cctstest="urn:un:unece:uncefact:documentation:2">
               <cctstest:ComponentType>ComponentType</cctstest:ComponentType>
               <cctstest:DictionaryEntryName>DictionaryEntryName</cctstest:DictionaryEntryName>
            </cctstest:Component>
          </type-documentation>
          <element-documentation>This element MUST be conveyed as the root
element in any instance document based on this Schema expression</element-documentation>
        </file>
      </dir>
    </dir>
  </configuration>
]]></programlisting>
  <para>
    The example for a user-defined extension and a user-defined additional
    document for UBL 2.1 is as follows:
  </para>
<programlisting><![CDATA[<configuration xmlns:xsd="http://www.w3.org/2001/XMLSchema"
               version="1">
  <abbreviations>
    <abbreviation>CV2</abbreviation>
    <abbreviation>ID</abbreviation>
    <abbreviation>URI</abbreviation>
    <abbreviation>UNDG</abbreviation>
    <abbreviation>UBL</abbreviation>
    <abbreviation>UUID</abbreviation>
    <abbreviation>XPath</abbreviation>
  </abbreviations>
  <types>
    <type>Amount</type>
    <type>Binary Object</type>
    <type>Code</type>
    <type>Date Time</type>
    <type>Date</type>
    <type>Graphic</type>
    <type>Identifier</type>
    <type>Indicator</type>
    <type>Measure</type>
    <type>Name</type>
    <type>Numeric</type>
    <type>Percent</type>
    <type>Picture</type>
    <type>Quantity</type>
    <type>Rate</type>
    <type>Sound</type>
    <type>Text</type>
    <type>Time</type>
    <type>Value</type>
    <type>Video</type>
  </types>
  <copyright position="end">
  Test copyright statement.
</copyright>
    <type-documentation>
      <ccts:Component xmlns:ccts="urn:un:unece:uncefact:documentation:2">
         <ccts:ComponentType>ComponentType</ccts:ComponentType>
         <ccts:DictionaryEntryName>DictionaryEntryName</ccts:DictionaryEntryName>
         <ccts:Definition>Definition</ccts:Definition>
         <ccts:Examples>Examples</ccts:Examples>
         <ccts:Cardinality>Cardinality</ccts:Cardinality>
         <ccts:ObjectClass>ObjectClass</ccts:ObjectClass>
         <ccts:PropertyTermQualifier>PropertyTermQualifier</ccts:PropertyTermQualifier>
         <ccts:PropertyTerm>PropertyTerm</ccts:PropertyTerm>
         <ccts:AssociatedObjectClass>AssociatedObjectClass</ccts:AssociatedObjectClass>
         <ccts:RepresentationTerm>RepresentationTerm</ccts:RepresentationTerm>
         <ccts:DataTypeQualifier>DataTypeQualifier</ccts:DataTypeQualifier>
         <ccts:DataType>DataType</ccts:DataType>
         <ccts:AlternativeBusinessTerms>AlternativeBusinessTerms</ccts:AlternativeBusinessTerms>
      </ccts:Component>
    </type-documentation>
    <dir name="xsd" runtime-name="xsdrt">
      <dir name="myext">
        <file type="XABIE" name="MyTimesheetExtension.xsd"
              prefix="mxd" sabie-prefix="mxa" sbbie-prefix="mxb"
              namespace="urn:X-MyCompany:xsd:MyTimesheetExtension">
        <comment>
  Library:           MyTimesheetExtension UBL extension demonstration
  Module:            %f
  Generated on:      %z
</comment>
        </file>
        <file type="SABIE" name="MyExtensionAggregateComponents.xsd"
              prefix="mxa"
              namespace="urn:X-MyCompany:xsd:MyTimesheetExtension:AggregateComponents">
        <comment>
  Library:           MyTimesheetExtension UBL extension demonstration
  Module:            %f
  Generated on:      %z
</comment>
        </file>
        <file type="SBBIE" name="MyExtensionBasicComponents.xsd"
              prefix="mxb"
              namespace="urn:X-MyCompany:xsd:MyTimesheetExtension:BasicComponents">
        <comment>
  Library:           MyTimesheetExtension UBL extension demonstration
  Module:            %f
  Generated on:      %z
</comment>
        </file>
      </dir>
      <dir name="mydoc">
        <file type="AABIE" name="MyRARequest.xsd" 
              abie="ReturnAuthorizationRequest"
              prefix="mydreq" sabie-prefix="mya" sbbie-prefix="myb"
              namespace="urn:X-MyCompany:xsd:MyRARequest">
        <comment>
  Library:           Return Authorization Request additional document demonstration
  Module:            %f
  Generated on:      %z
</comment>
        </file>
        <file type="AABIE" name="MyRAResponse.xsd"
              abie="ReturnAuthorizationResponse"
              prefix="mydres" sabie-prefix="mya" sbbie-prefix="myb"
              namespace="urn:X-MyCompany:xsd:MyRAResponse">
        <comment>
  Library:           Return Authorization Request additional document demonstration
  Module:            %f
  Generated on:      %z
</comment>
        </file>
        <file type="SABIE" name="MyRARequestResponseAggregateComponents.xsd"
              prefix="mya"
              namespace="urn:X-MyCompany:xsd:MyRARequestResponse:AggregateComponents">
        <comment>
  Library:           MyRARequestResponse additional document demonstration
  Module:            %f
  Generated on:      %z
</comment>
        </file>
        <file type="SBBIE" name="MyRARequestResponseBasicComponents.xsd"
              prefix="myb"
              namespace="urn:X-MyCompany:xsd:MyRARequestResponse:BasicComponents">
        <comment>
  Library:           MyRARequestResponse additional document demonstration
  Module:            %f
  Generated on:      %z
</comment>
        </file>
      </dir>
    </dir>
</configuration>]]></programlisting>

</xs:param>
<xsl:param name="configuration-uri" required="yes" as="xsd:anyURI"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Global variables and lookup tables</xs:title>
</xs:doc>

<xs:variable>
  <para>
    This is the access to the configuration file as a node tree.
  </para>
</xs:variable>
<xsl:variable name="gu:configMain" as="document-node()?"
              select="if( $configuration-uri ) 
                      then document($configuration-uri,/) else ()"/>

<xs:variable>
  <para>
    This is the access to the configuration file as a node tree.
  </para>
</xs:variable>
<xsl:variable name="gu:configOther" as="document-node()?"
              select="if( $common-config-uri ) 
                      then document($common-config-uri,/) else ()"/>

<xs:variable>
  <para>
    This is the access to the combination configuration file as a node tree.
  </para>
</xs:variable>
<xsl:variable name="gu:config" as="document-node()">
  <xsl:document>
    <combination>
      <xsl:for-each select="$gu:configMain,$gu:configOther">
        <xsl:for-each select="*">
          <!--copy the document element-->
          <xsl:copy>
            <xsl:copy-of select="@*"/>
            <!--this ensures relative URIs are relative to original
                configuration file and not this temporary tree-->
            <xsl:attribute name="xml:base" select="base-uri(.)"/>
            <!--copy the rest of the tree-->
            <xsl:copy-of select="node()"/>
          </xsl:copy>
        </xsl:for-each>
      </xsl:for-each>
    </combination>
  </xsl:document>
</xsl:variable>

<xs:variable>
  <para>
    This is the access to the genericode input file as a node tree.
  </para>
</xs:variable>
<xsl:variable name="gu:gc" as="document-node()" select="/"/>

<xs:variable>
  <para>
    This is the access to the genericode common file.
  </para>
</xs:variable>
<xsl:variable name="gu:gcOther" as="document-node()?"
              select="if( $common-gc-uri ) 
                      then document($common-gc-uri,/) else ()"/>

<xs:key>
  <para>
    Index the files in the directory structure of the configuration file.
  </para>
</xs:key>
<xsl:key name="gu:files" match="file|files" use="@type"/>

<xs:variable>
  <para>
    Handy reference to a sequence of possible spreadsheet column names.
    Although the UBL project uses "UBLName", another project may choose
    to use "ComponentName" as the corresponding column name.
  </para>
</xs:variable>
<xsl:variable name="gu:names" as="xsd:string+" 
              select="('UBLName','ComponentName')"/>

<xs:variable>
 <para>Determine the library model from all models in a genericode file.</para>
</xs:variable>
<xsl:variable name="gu:thisCommonLibraryModel" as="xsd:string?"
              select="gu:lookupCommonLibraryModel(/)"/>
<xs:variable>
 <para>
   Determine the library model from all models in the common genericode file
   for an extension/addition.
 </para>
</xs:variable>
<xsl:variable name="gu:otherCommonLibraryModel" as="xsd:string?"
              select="gu:lookupCommonLibraryModel($gu:gcOther)"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Utility templates and functions</xs:title>
</xs:doc>
  
<xs:function>
  <para>Return a row's column value based on a column reference</para>
  <xs:param name="row">
    <para>The row of the genericode file.</para>
  </xs:param>
  <xs:param name="col">
    <para>
      The column reference of the value in the row.  Note that multiple
      column references are allowed, but only one of the column references
      is allowed to match.  If the row matches more than one column name
      given, this will abend in a runtime error.
    </para>
  </xs:param>
</xs:function>
<xsl:function name="gu:col" as="element(SimpleValue)?">
  <xsl:param name="row" as="element(Row)"/>
  <xsl:param name="col" as="xsd:string+"/>
  <xsl:variable name="return" as="element(SimpleValue)*"
                select="$row/Value[@ColumnRef=$col]/SimpleValue"/>
  <xsl:if test="count($return) > 1">
    <xsl:message terminate="yes">
      <xsl:text>Data error: multiple genericode values in a single </xsl:text>
      <xsl:text>row for column reference</xsl:text>
      <xsl:if test="count($col)>1">s</xsl:if>: <xsl:text/>
      <xsl:value-of select="$col" separator=", "/> at <xsl:text/>
      <xsl:for-each select="$row/ancestor-or-self::*">
        <xsl:text/>/<xsl:value-of select="name(.)"/>
        <xsl:if test="self::Row">[<xsl:number/>]</xsl:if>
      </xsl:for-each>
    </xsl:message>
  </xsl:if>
  <xsl:sequence select="$return"/>
</xsl:function>
  
<xs:function>
  <para>
    Determine the one model that has more than one ABIE and assume that is
    the common library, since document models are allowed only to have a
    single ABIE.
  </para>
  <xs:param name="gc">
    <para>Which genericode file is being looked in?</para>
  </xs:param>
</xs:function>
<xsl:function name="gu:lookupCommonLibraryModel" as="xsd:string?">
  <xsl:param name="gc" as="document-node()?"/>
  <xsl:variable name="lookup" as="xsd:string*">
    <xsl:for-each-group group-by="gu:col(.,'ModelName')"
           select="$gc/*/SimpleCodeList/Row[gu:col(.,'ComponentType')='ABIE']">
        <xsl:if test="count(current-group())>1">
          <xsl:value-of select="current-grouping-key()"/>
        </xsl:if>
    </xsl:for-each-group>
  </xsl:variable>
  <!--when there are no such models (with more than one ABIE), assume that
      the first ABIE is in the common model-->
  <xsl:sequence select="if( $lookup ) then $lookup[1] else 
                        $gc/*/SimpleCodeList/Row
                 [gu:col(.,'ComponentType')='ABIE'][1]/gu:col(.,'ModelName')"/>
</xsl:function>
  
<xs:function>
  <para>Substitute key replacement strings in a given string</para>
  <xs:param name="name">
    <para>That value to use to substitute for the module name value</para>
  </xs:param>
  <xs:param name="string">
    <para>The string in which are found the substitution requests.</para>
  </xs:param>
</xs:function>
<xsl:function name="gu:substituteFields" as="xsd:string">
  <xsl:param name="name" as="xsd:string"/>
  <xsl:param name="string" as="xsd:string"/>
  <xsl:value-of>
    <xsl:analyze-string select="$string" regex="%(.)">
      <xsl:non-matching-substring>
        <xsl:value-of select="."/>
      </xsl:non-matching-substring>
      <xsl:matching-substring><xsl:value-of select="
        if(regex-group(1)='%') then '%' else
        if(regex-group(1)='f') then $name else
        if(regex-group(1)='n') then replace($name,'.*/','') else
        if(regex-group(1)='t') then format-dateTime( current-dateTime(), 
                                   '[Y0001]-[M01]-[D01] [H01]:[m01][Z]' ) else
        if(regex-group(1)='z') then format-dateTime( 
   adjust-dateTime-to-timezone(current-dateTime(),xsd:dayTimeDuration('PT0H')),
                                '[Y0001]-[M01]-[D01] [H01]:[m01]z' ) else
         ."/>
      </xsl:matching-substring>
    </xsl:analyze-string>
  </xsl:value-of>
</xsl:function>

</xsl:stylesheet>
