<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="../../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:gu="urn:X-gc2ublndr"
                exclude-result-prefixes="xs gu xsd"
                version="2.0">

<xsl:import href="Crane-commonndr.xsl"/>

<xs:doc info="$Id: Crane-gc2ublndr.xsl,v 1.33 2011/11/27 19:50:33 admin Exp $"
        filename="Crane-gc2ublndr.xsl" vocabulary="DocBook" internal-ns="gu">
  <xs:title>Translating genericode to UBL NDR XSD schemas</xs:title>
  <para> 
    This stylesheet reads as input a <emphasis>checked</emphasis> 
    OASIS genericode expression of the
    serialization of UBL NDR spreadsheets (see <xref linkend="ingc"/>) and a
    configuration file (see <xref linkend="invoke"/>). The output is the
    following set of W3C XML Schema XSD files and a single OASIS CVA file:
  </para>
  <itemizedlist>
    <listitem>
      a common library aggregate schema for all common library ABIEs
    </listitem>
    <listitem>
      a common library basic schema for all common library BBIEs
    </listitem>
    <listitem>
      a document schema for each document ABIE
    </listitem>
    <listitem>
      OASIS Context/Value Association files as needed for all code
      list qualifications
    </listitem>
  </itemizedlist>
  <note>
    <title>Important prerequisite</title>
      <para>
        The required checking is done by the <ulink
          url="readme-Crane-ublndrChecker.html"
            ><literal>Crane-ublndrChecker</literal></ulink> stylesheet before
        running this schema generator. This schema generator does not make all
        of the checks that the checker does, thus without checking, this
        generator may produce nonsensical results.
      </para>
  </note>
  <para>
    Additionally, this stylesheet can be invoked in order to create the
    necessary artefacts for either an extension for use under the 
    extension point, or as an additional document that shares the use of
    the common library.  In these cases the output is the following set
    of W3C XML Schema XSD files and OASIS CVA file:
  </para>
  <itemizedlist>
    <listitem>
      an extension aggregate schema for all extension ABIEs
    </listitem>
    <listitem>
      an extension library basic schema for all extension BBIEs
    </listitem>
    <listitem>
      an extension point schema or a document schema for each document ABIE
    </listitem>
    <listitem>
      OASIS Context/Value Association files as needed for all code
      list qualifications in the extension or document
    </listitem>
  </itemizedlist>
  <para>
    All output files are emitted relative to the base URI of the
    <emphasis>invoked</emphasis> output file (which, itself, is never 
    actually written to).  Therefore, supply a bogus filename in a bona fide
    output directory in order to specify where the output artefacts are 
    created.
  </para>
  <para>
    For example, the two invocations for the Saxon-HE 
<ulink url="http://saxon.sf.net"><literal>http://saxon.sf.net</literal></ulink>
    processor would be along the lines of the following in order to generate
    a set of directories under the current directory,
    first for the base set of UBL artefacts and second for the signature
    extension artefacts (as is created when invoking
    <literal>createUBL.sh</literal> or <literal>createUBL.bat</literal>):
  </para>
  <blockquote>
    <programlisting>java -jar ../saxon9he.jar -s:mod/UBL-Entities-2.1-PRD2.gc 
     -xsl:../Crane-gc2ublndr.xsl -o:junk.out qdt-as-cva=yes 
     configuration-uri=../config-ubl-2.1.xml
java -jar ../saxon9he.jar -s:mod/UBL-Signature-Entities-2.1-PRD2.gc
     -xsl:../Crane-gc2ublndr.xsl -o:junk.out qdt-as-cva=yes 
     configuration-uri=../config-ubl-2.1-ext.xml xabie-prefix=sig 
     common-gc-uri=UBL-Entities-2.1-PRD2.gc</programlisting>
  </blockquote>
  <para>
    ... where the <literal>junk.out</literal> file is never actually created
    as it is used only to indicate the directory in which the output belongs.
  </para>
  <para>
    The two invocations would be along the lines of the following in order to
    generate customized construct artefacts, first for an extension for under
    the extension point, and the second two for an additional document sharing
    the common library (as is created when invoking
    <literal>createMyExt.sh</literal> or <literal>createMyExt.bat</literal>):
  </para>
  <blockquote>
    <programlisting>
java -jar ../saxon9he.jar -s:mod/MyTimesheetExtension-Entities.gc 
     -xsl:../Crane-gc2ublndr.xsl -o:junk.out qdt-as-cva=yes 
     configuration-uri=../config-myext.xml xabie-prefix=mxd 
     common-gc-uri=UBL-Entities-2.1-PRD2.gc 
     common-config-uri=../config-ubl-2.1.xml
java -jar ../saxon9he.jar -s:mod/MyRARequestResponse-Entities.gc
     -xsl:../Crane-gc2ublndr.xsl -o:junk.out qdt-as-cva=yes
     configuration-uri=../config-myext.xml aabie-prefix=mydreq
     common-gc-uri=UBL-Entities-2.1-PRD2.gc
     common-config-uri=../config-ubl-2.1.xml
java -jar ../saxon9he.jar -s:mod/MyRARequestResponse-Entities.gc
     -xsl:../Crane-gc2ublndr.xsl -o:junk.out qdt-as-cva=yes
     configuration-uri=../config-myext.xml aabie-prefix=mydres
     common-gc-uri=UBL-Entities-2.1-PRD2.gc
     common-config-uri=../config-ubl-2.1.xml</programlisting>
  </blockquote>
  <para>
    See the paper <ulink url="CreatingExtensionsWithUBLNDR.html"
      ><literal>CreatingExtensionsWithUBLNDR.html</literal></ulink> for
    more details.
  </para>
  <section>
    <title>Copyright and disclaimer</title>
<programlisting>Copyright (C) - Crane Softwrights Ltd.
              - <ulink url="http://www.CraneSoftwrights.com/links/res-dev.htm">http://www.CraneSoftwrights.com/links/res-dev.htm</ulink>

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.
- The name of the author may not be used to endorse or promote products
  derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Note: for your reference, the above is the "Modified BSD license", this text
      was obtained 2003-07-26 at <ulink
url="http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5">http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5</ulink>

THE AUTHOR MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS CODE FOR ANY
PURPOSE.</programlisting>
    </section>
</xs:doc>

<!--========================================================================-->
<xs:doc>
  <xs:title>Invocation parameters</xs:title>
</xs:doc>
  
<xs:param ignore-ns="yes">
  <para>
    Indicate using only the values "yes" or "no" if qualified data types are
    entirely implemented using an OASIS Context/Value Association file.
    The value "yes" will inhibit all data type qualifications using a formal
    Qualified Data Type, instead using an Unqualified Data Type by assuming
    that a CVA file will take the responsibility of checking data type 
    qualifications.
  </para>
  <para>
    The OASIS Context/Value Association specification is available at
    <ulink url="http://docs.oasis-open.org/codelist/ContextValueAssociation/">
      <literal>﻿http://docs.oasis-open.org/codelist/ContextValueAssociation/</literal>
    </ulink>.
  </para>
</xs:param>
<xsl:param name="qdt-as-cva" required="yes" as="xsd:string"/>

<xs:param ignore-ns="yes">
  <para>
    Specify this to indicate which extension schema fragment and related
    aggregate and basic fragments are to be generated in lieu of other 
    processing.
  </para>
</xs:param>
<xsl:param name="xabie-prefix" as="xsd:string?" select="()"/>

<xs:param ignore-ns="yes">
  <para>
    Specify this to indicate which additional document schema fragment and
    related aggregate and basic fragments are to be generated in lieu of other 
    processing.
  </para>
</xs:param>
<xsl:param name="aabie-prefix" as="xsd:string?" select="()"/>

  <xs:output>
  <para>Indent the output assuming humans will be reading this.</para>
</xs:output>
<xsl:output indent="yes"/>
  
<xs:key>
  <para>
    Index the genericode file for all ABIEs, using the common library model
    name as the lookup for common ABIEs, and using the ABIE name as the lookup
    for document ABIEs.
  </para>
</xs:key>
<xsl:key name="gu:abie-by-name-or-model" 
         match="/*/SimpleCodeList/Row[gu:col(.,'ComponentType')='ABIE']"
         use="if( gu:col(.,'ModelName')=$gu:thisCommonLibraryModel )
              then $gu:thisCommonLibraryModel 
              else gu:col(.,$gu:names)"/>

<xs:key>
  <para>
    Index the genericode file for all BBIEs.
  </para>
</xs:key>
<xsl:key name="gu:bbie-by-name" 
         match="/*/SimpleCodeList/Row[gu:col(.,'ComponentType')='BBIE']"
         use="gu:col(.,$gu:names)"/>

<xs:key>
  <para>
    Index the genericode file for all ABIEs.
  </para>
</xs:key>
<xsl:key name="gu:abie-by-name" 
         match="/*/SimpleCodeList/Row[gu:col(.,'ComponentType')='ABIE']"
         use="gu:col(.,$gu:names)"/>

<xs:key>
  <para>
    Index the genericode file for all business information entities by type
  </para>
</xs:key>
<xsl:key name="gu:bie-by-type" match="/*/SimpleCodeList/Row"
         use="gu:col(.,'ComponentType')"/>

<xs:key>
  <para>
    Index the genericode file for all BIEs for a given ABIE
  </para>
</xs:key>
<xsl:key name="gu:bie-by-abie" 
         match="/*/SimpleCodeList/Row[gu:col(.,'ComponentType')!='ABIE']"
         use="preceding-sibling::Row[gu:col(.,'ComponentType')='ABIE'][1]/
              gu:col(.,$gu:names)"/>

<xs:key>
  <para>
    Index the genericode file for all ASBIEs for a given ABIE
  </para>
</xs:key>
<xsl:key name="gu:asbie-by-abie" 
         match="/*/SimpleCodeList/Row[gu:col(.,'ComponentType')='ASBIE']"
         use="preceding-sibling::Row[gu:col(.,'ComponentType')='ABIE'][1]/
              gu:col(.,$gu:names)"/>

<xs:key>
  <para>
    Index the genericode file for all BBIEs by Data Type.
  </para>
</xs:key>
<xsl:key name="gu:bbie-by-data-type" 
         match="/*/SimpleCodeList/Row[gu:col(.,'ComponentType')='BBIE']"
         use="gu:col(.,'DataType')"/>

<xs:variable>
  <para>
    Summary list of documents names is found by the name of every ABIE
    that is not in the common library.
  </para>
</xs:variable>
<xsl:variable name="gu:documentABIEs" as="element(SimpleValue)*">
  <xsl:for-each-group select="/*/SimpleCodeList/
                         Row[gu:col(.,'ModelName')!=$gu:thisCommonLibraryModel]
                            [gu:col(.,'ComponentType')='ABIE']/
                         gu:col(.,$gu:names)" group-by=".">
    <xsl:sequence select="."/>
  </xsl:for-each-group>
</xsl:variable>
              
<xs:variable>
  <para>
    Summary list of documents names is found by the name of every ABIE
    that is not in the common library.
  </para>
</xs:variable>
<xsl:variable name="gu:libraryABIEs" as="element(SimpleValue)*">
  <xsl:for-each-group select="/*/SimpleCodeList/
                         Row[gu:col(.,'ModelName')=$gu:thisCommonLibraryModel]
                            [gu:col(.,'ComponentType')='ABIE']/
                         gu:col(.,$gu:names)" group-by=".">
    <xsl:sequence select="."/>
  </xsl:for-each-group>
</xsl:variable>
              
<xs:variable>
  <para>
    Handy prefix definition so as not to keep looking it up.
  </para>
</xs:variable>
<xsl:variable name="gu:cbbiePrefixColon" as="xsd:string"
              select="concat(key('gu:files','CBBIE',$gu:config)/@prefix,':')"/>
              
<xs:variable>
  <para>
    Handy prefix definition so as not to keep looking it up.
  </para>
</xs:variable>
<xsl:variable name="gu:cabiePrefixColon" as="xsd:string"
              select="concat(key('gu:files','CABIE',$gu:config)/@prefix,':')"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Overall stylesheet flow</xs:title>
</xs:doc>

<xs:template>
  <para>
    In the absence of a schema for the configuration file, first check
    simple assumptions regarding the configuration file.
  </para>
  <para>
    Then each of the individual document ABIE files are created, then the 
    common library ABIE files, then the CVA file.
  </para>
</xs:template>
<xsl:template match="/">
  <!--check assumptions regarding the configuration file-->
  <xsl:variable name="messages">
    <xsl:choose>
      <xsl:when test="not(normalize-space($configuration-uri))">
        <xsl:text>Configuration uri cannot be empty.</xsl:text>
      </xsl:when>
      <xsl:when test="not($gu:configMain)">
        <xsl:text>Cannot open specified configuration file "</xsl:text> 
        <xsl:value-of select="$configuration-uri"/>
        <xsl:text>" resolved relative to the input genericode file: </xsl:text>
        <xsl:value-of select="document-uri(/)"/>
      </xsl:when>
      <xsl:when test="$common-config-uri and not($gu:configOther)">
        <xsl:text>Cannot open specified common configuration file "</xsl:text> 
        <xsl:value-of select="$common-config-uri"/>
        <xsl:text>" resolved relative to the input genericode file: </xsl:text>
        <xsl:value-of select="document-uri(/)"/>
      </xsl:when>
      <xsl:when test="($xabie-prefix or $aabie-prefix) and not($gu:gcOther)">
        <xsl:text>Cannot open specified common genericode file "</xsl:text> 
        <xsl:value-of select="$configuration-uri"/>
        <xsl:text>" resolved relative to the input genericode file: </xsl:text>
        <xsl:value-of select="document-uri(/)"/>
      </xsl:when>
      <xsl:otherwise>
        <!--validate the contents of the configuration file-->
        <xsl:call-template name="gu:validateInputs"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:if test="normalize-space($messages)">
    <xsl:message terminate="yes" select="$messages"/>
  </xsl:if>
  <!--note that files in the "other" configuration file are not created-->
  <xsl:variable name="makeConfig" select="$gu:config/*/configuration[1]"/>
  <!--the CVA files are processed first in case of problem finding skeleton -->
  <xsl:apply-templates 
                 select="key('gu:files','CVA',$makeConfig)"/>
  <!--first assume no runtime output, then request runtime output if needed,
      for each of the files that generate XSD schemas-->
  <xsl:for-each select="false(),
                    if( $makeConfig//dir[@runtime-name] ) then true() else ()">
    <!--determine if working with an extension/addition or not-->
    <xsl:choose>
      <xsl:when test="$xabie-prefix">
        <!--yes, working with an extension, so need to find which
            one and process those components related to that one-->
        <xsl:apply-templates select="key('gu:files','XABIE',$makeConfig)
                                     [@prefix=$xabie-prefix]">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'XABIE'"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="key('gu:files','SABIE',$makeConfig)
          [@prefix=key('gu:files','XABIE',$makeConfig)[@prefix=$xabie-prefix]/
                   @sabie-prefix]">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'SABIE'"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="key('gu:files','SBBIE',$makeConfig)
          [@prefix=key('gu:files','XABIE',$makeConfig)[@prefix=$xabie-prefix]/
                   @sbbie-prefix]">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'SBBIE'"/>
        </xsl:apply-templates>
      </xsl:when>
      <xsl:when test="$aabie-prefix">
        <!--yes, working with an addition, so need to find which
            one and process those components related to that one-->
        <xsl:apply-templates select="key('gu:files','AABIE',$makeConfig)
                                     [@prefix=$aabie-prefix]">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'AABIE'"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="key('gu:files','SABIE',$makeConfig)
          [@prefix=key('gu:files','AABIE',$makeConfig)[@prefix=$aabie-prefix]/
                   @sabie-prefix]">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'SABIE'"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="key('gu:files','SBBIE',$makeConfig)
          [@prefix=key('gu:files','AABIE',$makeConfig)[@prefix=$aabie-prefix]/
                   @sbbie-prefix]">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'SBBIE'"/>
        </xsl:apply-templates>
      </xsl:when>
      <xsl:otherwise>
        <!--no, working with the base set of business objects-->
        <xsl:apply-templates select="key('gu:files','DABIE',$makeConfig)">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'DABIE'"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="key('gu:files','CABIE',$makeConfig)">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'CABIE'"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="key('gu:files','CBBIE',$makeConfig)">
          <xsl:with-param name="runtime" tunnel="yes" select="."/>
          <xsl:with-param name="schemaType" tunnel="yes" select="'CBBIE'"/>
        </xsl:apply-templates>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>
  
<xs:template>
  <para>
    A single XSD file is being created
  </para>
  <xs:param name="runtime">
    <para>
      An indication if the file being created is for a runtime directory
      as compared to a directory of fully-documented constructs.
    </para>
  </xs:param>
</xs:template>
<xsl:template match="configuration//file">
  <xsl:param name="runtime" tunnel="yes" as="xsd:boolean"/>
  <xsl:variable name="module" 
        select="gu:relativeConfigURI( ., ancestor::configuration, $runtime )"/>
  <xsl:message select="'Creating',$module,'...'"/>
  <xsl:variable name="moduleABIEs"
                select="if( @abie )
                        then $gu:documentABIEs[.=current()/@abie]
                        else if( @type=('DABIE','XABIE','AABIE') )
                        then $gu:documentABIEs[1]
                        else (:'CABIE','SABIE','CBBIE','SBBIE':)
                             ( $gu:documentABIEs, $gu:libraryABIEs )"/>
  <xsl:result-document href="{$module}">
    <xsl:call-template name="gu:schemaWrite">
      <xsl:with-param name="module" select="$module"/>
      <xsl:with-param name="moduleABIEs" tunnel="yes" select="$moduleABIEs"/>
      <xsl:with-param name="moduleBBIEs" tunnel="yes"
                     select="$moduleABIEs/key('gu:bie-by-abie',.,$gu:gc)
                                          [gu:col(.,'ComponentType')='BBIE']/
                                          gu:col(.,$gu:names)"/>
      <xsl:with-param name="namespace" 
                      select="gu:substituteFields($module,@namespace)"/>
    </xsl:call-template>
  </xsl:result-document>
</xsl:template>

<xs:template>
  <para>
    A number of files are being created, but not including those that are
    created individually.
  </para>
  <xs:param name="runtime">
    <para>
      An indication if the file being created is for a runtime directory
      as compared to a directory of fully-documented constructs.
    </para>
  </xs:param>
</xs:template>
<xsl:template match="configuration//files">
  <xsl:param name="runtime" tunnel="yes" as="xsd:boolean"/>
  <xsl:variable name="thisFile" select="."/>
  <!--determine which names are already being written individually-->
  <xsl:variable name="singleDocumentNames" as="xsd:string*"
                select="key('gu:files',@type,$gu:config)/@abie"/>
  <!--write out all those models not included in the names already written-->
  <xsl:for-each select="$gu:documentABIEs[not(.=$singleDocumentNames)]">
    <xsl:variable name="thisDocumentABIE" select="."/>
    <!--reposition to the node in the configuration tree-->
    <xsl:for-each select="$thisFile">
      <xsl:variable name="module" 
       select="gu:substituteFields($thisDocumentABIE,( gu:relativeConfigURI( .,
                                      ancestor::configuration, $runtime) ) )"/>
      <xsl:message select="'Creating',$module,'...'"/>
      <xsl:result-document href="{$module}">
        <xsl:call-template name="gu:schemaWrite">
          <xsl:with-param name="module" select="$module"/>
          <xsl:with-param name="moduleABIEs" tunnel="yes" 
                          select="$thisDocumentABIE"/>
          <xsl:with-param name="moduleBBIEs" tunnel="yes"
                        select="key('gu:bie-by-abie',$thisDocumentABIE,$gu:gc)
                                [gu:col(.,'ComponentType')='BBIE']/
                                gu:col(.,$gu:names)"/>
          <xsl:with-param name="namespace" 
                   select="gu:substituteFields($thisDocumentABIE,@namespace)"/>
          <xsl:with-param name="schemaType" tunnel="yes" select="@type"/>
        </xsl:call-template>
      </xsl:result-document>
    </xsl:for-each>
  </xsl:for-each>
</xsl:template>

<xs:template>
  <para>
    Write out the OASIS Context/Value Association (CVA) file reflecting
    model information.
  </para>
  <para>
    It is assumed that this artefact does not live in any runtime directory
    and thus is not looking for any <literal>runtime-name=</literal> 
    attributes in the configuration file.
  </para>
</xs:template>
<xsl:template match="configuration//file[@type='CVA']" priority="1">
  <xsl:variable name="skeleton" select="document(@skeleton-uri)"/>
  <xsl:variable name="module" 
         select="gu:relativeConfigURI( ., ancestor::configuration, false() )"/>
  <xsl:message select="'Creating',$module,'...'"/>
  <xsl:result-document href="{$module}">
    <xsl:apply-templates mode="gu:CVAWrite" select="$skeleton"/>
  </xsl:result-document>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>Schema generator</xs:title>
</xs:doc>
  
<xs:template>
  <para>
    Create a schema for the configuration file entry at the current node.
  </para>
  <xs:param name="module">
    <para>The schema module being written (e.g. UBL-Invoice-2.1.xsd).</para>
  </xs:param>
  <xs:param name="namespace">
    <para>The target namespace of the schema.</para>
  </xs:param>
  <xs:param name="schemaType">
    <para>
      The type of schema being written (CABIE, CBBIE, DABIE, SABIE, SBBIE,
      XABIE, AABIE ).</para>
  </xs:param>
  <xs:param name="moduleABIEs">
    <para>
      The ABIEs under influence of this schema fragment.
    </para>
  </xs:param>
  <xs:param name="moduleBBIEs">
    <para>
      The BBIEs under influence of this schema fragment.
    </para>
  </xs:param>
  <xs:param name="runtime">
    <para>
      An indication if the file being created is for a runtime directory
      as compared to a directory of fully-documented constructs.
    </para>
  </xs:param>
</xs:template>
<xsl:template name="gu:schemaWrite">
  <xsl:param name="module" as="xsd:string"/>
  <xsl:param name="namespace" as="xsd:string"/>
  <xsl:param name="schemaType" tunnel="yes" as="xsd:string"/>
  <xsl:param name="runtime" tunnel="yes" as="xsd:boolean"/>
  <xsl:param name="moduleABIEs" tunnel="yes" as="element(SimpleValue)*"/>
  <xsl:param name="moduleBBIEs" tunnel="yes" as="element(SimpleValue)*"/>
  <xsl:variable name="thisFile" select="."/>
  <xsl:text>
</xsl:text>
  <xsl:for-each select="(comment,ancestor::configuration/comment)[1]">
    <!--put out either the specific comment or the generic comment-->
    <xsl:comment>
      <xsl:value-of select="gu:substituteFields($module,.)"/>
    </xsl:comment>
    <xsl:text>
</xsl:text>
  </xsl:for-each>
  <!--put out either the specific copyright or the generic copyright-->
  <xsl:for-each 
  select="(copyright,ancestor::configuration/copyright)[1][@position='start']">
    <xsl:comment> ===== Copyright Notice ===== </xsl:comment>
    <xsl:text>
</xsl:text>
    <xsl:comment select="."/>
  </xsl:for-each>
  <!--
       [GXS4] 8.2
       All XSD constructs in UBL schema and schema modules MUST contain the
       following namespace declaration on the xsd:schema element:
        xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  -->
  <xsd:schema targetNamespace="{$namespace}"
              elementFormDefault="qualified"
              attributeFormDefault="unqualified"
              version="{ancestor::configuration/@version}">
  <xsl:variable name="otherParticipatingFiles" as="element(file)*">
    <!--next check need for the supplemental ABIEs-->
    <xsl:if test="$schemaType='XABIE' and
                  $moduleABIEs[not(gu:isInOtherLibrary(.))]">
      <xsl:sequence select="key('gu:files','SABIE',$gu:config)
      [@prefix=key('gu:files','XABIE',$gu:config)[@prefix=$xabie-prefix]/
               @sabie-prefix]"/>
    </xsl:if>
    <xsl:if test="$schemaType='AABIE' and
                  $moduleABIEs[not(gu:isInOtherLibrary(.))]">
      <xsl:sequence select="key('gu:files','SABIE',$gu:config)
      [@prefix=key('gu:files','AABIE',$gu:config)[@prefix=$aabie-prefix]/
               @sabie-prefix]"/>
    </xsl:if>
    <!--next check need for the supplemental BBIEs-->
    <xsl:if test="$schemaType='SABIE' and 
                  $moduleBBIEs[not(gu:isInOtherLibrary(.))]">
      
    </xsl:if>
    <xsl:if test="$schemaType=('XABIE','SABIE') and $xabie-prefix and
                  $moduleBBIEs[not(gu:isInOtherLibrary(.))]">
      <xsl:sequence select="key('gu:files','SBBIE',$gu:config)
      [@prefix=key('gu:files','XABIE',$gu:config)[@prefix=$xabie-prefix]/
               @sbbie-prefix]"/>
    </xsl:if>
    <xsl:if test="$schemaType=('AABIE','SABIE') and $aabie-prefix and
                  $moduleBBIEs[not(gu:isInOtherLibrary(.))]">
      <xsl:sequence select="key('gu:files','SBBIE',$gu:config)
      [@prefix=key('gu:files','AABIE',$gu:config)[@prefix=$aabie-prefix]/
               @sbbie-prefix]"/>
    </xsl:if>
    <!--next check need for the common library ABIEs-->
    <xsl:if test="( $schemaType='DABIE' and
                    $moduleABIEs[not(gu:isInOtherLibrary)] ) or
                  $moduleABIEs[gu:isInOtherLibrary(.)] or
                  (:check ASBIEs for pointing to common library:)
                  $moduleABIEs/key('gu:asbie-by-abie',.,$gu:gc)/
                               gu:col(.,'AssociatedObjectClass')
                               [gu:isInOtherLibrary(.)]">
      <xsl:sequence select="key('gu:files','CABIE',$gu:config)"/>
    </xsl:if>
    <!--next check need for the common library BBIEs-->
    <xsl:if test="( $schemaType='DABIE' and
                    $moduleBBIEs[not(gu:isInOtherLibrary)] ) or
                  ( $schemaType='CABIE' and $moduleBBIEs ) or
                  ( not( $schemaType=('SBBIE','CBBIE') ) and
                    $moduleBBIEs[gu:isInOtherLibrary(.)] )">
      <xsl:sequence select="key('gu:files','CBBIE',$gu:config)"/>
    </xsl:if>
    <xsl:if test="$schemaType=('CBBIE','SBBIE')">
      <!--might need to make reference to the data types-->
      <xsl:sequence select="key('gu:files','QDT',$gu:config),
                            key('gu:files','UDT',$gu:config)"/>
    </xsl:if>
    <!--document-level ABIEs are augmented with extensions-->
    <xsl:if test="$schemaType=('DABIE','AABIE')">
      <xsl:sequence select="key('gu:files','EXT',$gu:config)"/>
    </xsl:if>
  </xsl:variable>
  
  <!--the module's namespace is needed for local references-->
  <xsl:namespace name="" select="$namespace"/>
  <!--add namespaces for all participating files-->
  <xsl:for-each select="(:first check self-referencing common aggregates:)
                        .[$schemaType=('CABIE','SABIE')],
                        (:then add in all of the participating files:)
                        $otherParticipatingFiles">
    <xsl:namespace name="{@prefix}" select="@namespace"/>
  </xsl:for-each>
  <!--add namespaces from any descendants-->
  <xsl:copy-of select=".//namespace::*"/>
  <!--documentation namespaces may be needed-->
  <xsl:if test="not($runtime) and not($schemaType=('CBBIE','SBBIE'))">
    <xsl:copy-of select="(type-documentation,
                          ancestor::configuration/type-documentation)[1]//
                         namespace::*"/>
    <xsl:copy-of select="(element-documentation,
                          ancestor::configuration/element-documentation)[1]//
                         namespace::*"/>
  </xsl:if>

  <!--deal with import statements-->
  <xsl:if test="$otherParticipatingFiles or imports/node()">
    <xsl:call-template name="gu:sectionComment">
      <xsl:with-param name="section">Imports</xsl:with-param>
    </xsl:call-template>
    <xsl:for-each select="$otherParticipatingFiles">
      <xsl:element name="xsd:import">
        <xsl:attribute name="namespace" select="@namespace"/>
        <!--
       [GXS15] 8.11
       Each xsd:schemaLocation attribute declaration MUST contain a 
       system-resolvable URL, which at the time of release from OASIS shall be
       a relative URL referencing the location of the schema or schema module
       in the release package.
        -->
        <xsl:attribute name="schemaLocation"
                       select="gu:relativeConfigURI(.,$thisFile,$runtime)"/>
      </xsl:element>
    </xsl:for-each>
    <!--the config file declaration may include some import constructs to add-->
    <xsl:copy-of select="imports/node()"/>
  </xsl:if>

  <xsl:call-template name="gu:elementsSection"/>

  <xsl:call-template name="gu:typesSection"/>

  </xsd:schema>
  <!--put out either the specific copyright or the generic copyright-->
  <xsl:for-each 
    select="(copyright,ancestor::configuration/copyright)[1][@position='end']">
    <xsl:comment> ===== Copyright Notice ===== </xsl:comment>
    <xsl:comment select="."/>
  </xsl:for-each>
</xsl:template>
  
<xs:template>
  <para>
    The elements section declares all of the elements and the types they have.
  </para>
  <xs:param name="moduleABIEs">
    <para>The ABIEs under influence of this schema fragment.</para>
  </xs:param>
  <xs:param name="moduleBBIEs">
    <para>
      The BBIEs under influence of this schema fragment.
    </para>
  </xs:param>
  <xs:param name="schemaType">
    <para>The type of schema being written (CABIE, CBBIE, DABIE).</para>
  </xs:param>
  <xs:param name="runtime">
    <para>
      An indication if the file being created is for a runtime directory
      as compared to a directory of fully-documented constructs.
    </para>
  </xs:param>
</xs:template>
<xsl:template name="gu:elementsSection">
  <xsl:param name="moduleABIEs" tunnel="yes" as="element(SimpleValue)*"/>
  <xsl:param name="moduleBBIEs" tunnel="yes" as="element(SimpleValue)*"/>
  <xsl:param name="schemaType" tunnel="yes" as="xsd:string"/>
  <xsl:param name="runtime" tunnel="yes" as="xsd:boolean"/>
  <xsl:variable name="thisFile" select="."/>

  <xsl:call-template name="gu:sectionComment">
    <xsl:with-param name="section">Element Declarations</xsl:with-param>
  </xsl:call-template>
  <xsl:choose>
    <xsl:when test="$schemaType=('DABIE','XABIE','AABIE')">
      <!--
       [RED2] 3.1.2
       The root element MUST be the only global element declared in the 
       document schema.
       [ELD2] 3.3.1
       All element declarations MUST be global.
       [ELD3] 6.2.1
       For every class and property identified in the UBL model, a global 
       element bound to the corresponding xsd:complexType MUST be declared.
      -->
      <xsl:element name="xsd:element">
        <xsl:attribute name="name" select="$moduleABIEs"/>
          <!--
               [GTD1] 6.1.1
               All types MUST be named.
          -->
        <xsl:attribute name="type" select="concat($moduleABIEs,'Type')"/>
        <xsl:for-each select="(element-documentation,
                               ancestor::configuration/element-documentation)
                              [1][not($runtime)]">
          <xsl:element name="xsd:annotation">
            <xsl:element name="xsd:documentation">
              <!--[MDC2] 3.2.1.1.3
    XML mixed content MUST NOT be used except where contained in an 
    xsd:documentation element.-->
              <xsl:copy-of select="node()" copy-namespaces="no"/>
            </xsl:element>
          </xsl:element>
        </xsl:for-each>
      </xsl:element>
    </xsl:when>
    <xsl:when test="$schemaType=('CABIE','SABIE')">
      <xsl:variable name="nameTypePairs" as="element(pair)*">
        <!--only work with the ABIEs that are not in the other library and
            are not, themselves, a document-level ABIE-->
        <xsl:for-each select="$moduleABIEs[$schemaType='CABIE' or
                                           not(gu:isInOtherLibrary(.))]
                                          [not(.=$gu:documentABIEs)]">
          <pair>
            <name><xsl:value-of select="."/></name>
            <xsl:text>&#xd;</xsl:text>
            <type><xsl:value-of select="."/></type>
          </pair>
        </xsl:for-each>
        <!--check each ASBIE whether it is already an ABIE or its ABIE is
            in the other library-->
        <!--
       [ELD4] 6.2.2
       When a CCTS ASBIE is unqualified, it is bound via reference to the 
       global CCTS ABIE element with which it is associated.
       [ELD11] 6.2.2
       When a CCTS ASBIE is qualified, a new element MUST be declared and bound
       to the xsd:complexType of its associated CCTS ABIE.
        -->
        <xsl:for-each select="$moduleABIEs/key('gu:asbie-by-abie',.,$gu:gc)/
                              gu:col(.,$gu:names)
                         [$schemaType='CABIE' or
                         ( not(gu:isInOtherLibrary(gu:col(../..,$gu:names)))
                           and not(.=$moduleABIEs) )]">
          <!--create a pair whenever the ASBIE is not in the other library-->
          <pair>
            <name><xsl:value-of select="."/></name>
            <xsl:text>&#xd;</xsl:text>
            <type>
              <!--need to determine if the type is local or common-->
              <xsl:for-each select="
                     translate(gu:col(../..,'AssociatedObjectClass'),' ','')">
                <xsl:if test="$gu:gcOther/key('gu:bie-by-type','ABIE',.)
                            [gu:col(.,'ModelName')=$gu:otherCommonLibraryModel]
                            [gu:col(.,$gu:names)=current()]">
                  <!--the type is common, so the prefix is needed-->
                  <xsl:value-of select="$gu:cabiePrefixColon"/>
                </xsl:if>
                <xsl:value-of select="."/>
              </xsl:for-each>
            </type>
          </pair>
        </xsl:for-each>
      </xsl:variable>
      <!--make all the necessary declarations for the elements-->
      <xsl:for-each-group select="$nameTypePairs" group-by=".">
        <xsl:sort select="."/>
        <xsl:element name="xsd:element">
          <!--
       [CTN1] 5.2.1
       A UBL xsd:complexType name based on a CCTS ABIE MUST be the CCTS 
       Dictionary Entry Name with the separators removed and with the "Details"
       suffix replaced with "Type".
       [ELD2] 3.3.1
       All element declarations MUST be global.
       [ELN1] 5.3.1
       A UBL global element name based on a CCTS ABIE MUST be the same as the 
       name of the corresponding xsd:complexType to which it is bound, with the
       word "Type" removed.
          -->
          <xsl:attribute name="name" select="name"/>
          <xsl:attribute name="type" select="concat(type,'Type')"/>
        </xsl:element>
      </xsl:for-each-group>
    </xsl:when>
    <xsl:when test="$schemaType=('CBBIE','SBBIE')">
      <xsl:for-each-group select="$moduleBBIEs" group-by=".">
        <xsl:sort select="."/>
        <xsl:if test="$schemaType='CBBIE' or
                      not(gu:isInOtherLibrary(.))">
          <!--
       [ELD2] 3.3.1
       All element declarations MUST be global.
       [CTN2] 5.2.2
       A UBL xsd:complexType name based on a CCTS BBIE Property MUST be the 
       CCTS Dictionary Entry Name shared Property Term and its qualifiers and
       the Representation Term of the BBIE with the separators removed and with
       the "Type" suffix appended after the Representation Term.
       [ELN2] 5.3.2
       A UBL global element name based on a CCTS BBIE Property MUST be the same
       as the name of the corresponding xsd:complexType to which it is bound, 
       with the word "Type" removed.
          -->
          <xsl:element name="xsd:element">
            <xsl:attribute name="name" select="."/>
            <xsl:attribute name="type" select="concat(.,'Type')"/>
          </xsl:element>
        </xsl:if>
      </xsl:for-each-group>
    </xsl:when>
  </xsl:choose>
</xsl:template>
  
<xs:template>
  <para>
    The types section defines the type of each element.
  </para>
  <xs:param name="moduleABIEs">
    <para>The ABIEs under influence of this schema fragment.</para>
  </xs:param>
  <xs:param name="moduleBBIEs">
    <para>
      The BBIEs under influence of this schema fragment.
    </para>
  </xs:param>
  <xs:param name="schemaType">
    <para>The type of schema being written (CABIE, CBBIE, DABIE).</para>
  </xs:param>
  <xs:param name="runtime">
    <para>
      as compared to a directory of fully-documented constructs.
      An indication if the file being created is for a runtime directory
    </para>
  </xs:param>
</xs:template>
<xsl:template name="gu:typesSection">
  <xsl:param name="schemaType" tunnel="yes" as="xsd:string"/>
  <xsl:param name="runtime" tunnel="yes" as="xsd:boolean"/>
  <xsl:param name="moduleABIEs" tunnel="yes" as="element(SimpleValue)*"/>
  <xsl:param name="moduleBBIEs" tunnel="yes" as="element(SimpleValue)*"/>
  <xsl:variable name="thisFile" select="."/>
  <xsl:variable name="documentation"
        select="(type-documentation,ancestor::configuration/type-documentation)
                [1][not($runtime)]"/>
  <xsl:choose>
    <xsl:when test="$schemaType=('DABIE','CABIE','XABIE','AABIE','SABIE')">
      <xsl:variable name="thisCBCprefixColon"
                    select="if( $schemaType = ('DABIE','CABIE') ) 
                            then $gu:cbbiePrefixColon
                            else if( $schemaType = ('XABIE','AABIE') )
                            then concat( @sbbie-prefix, ':' )
                            else concat( 
                    (//file)[@sabie-prefix=current()/@prefix][1]/@sbbie-prefix,
                                        ':' )"/>
      <xsl:variable name="thisCACprefixColon"
                    select="if( $schemaType = 'DABIE' ) 
                            then $gu:cabiePrefixColon
                            else if( $schemaType = ('XABIE','AABIE') )
                            then concat( @sabie-prefix, ':' )
                            else ''"/>
      <xsl:call-template name="gu:sectionComment">
        <xsl:with-param name="section" 
          select="'Type Definitions',
                  'Aggregate Business Information Entity Type Definitions'"/>
      </xsl:call-template>
      <xsl:for-each select="$moduleABIEs
        [not( $schemaType=('CABIE','SABIE') and ( .=$gu:documentABIEs ) ) ]
                                          [not(gu:isInOtherLibrary(.))]/../..">
        <xsl:sort select="gu:col(.,$gu:names)"/>
        <xsl:variable name="thisABIE" select="."/>
        <!--
          [CTD1] 6.1.3
          For every class identified in the UBL model, a named xsd:complexType 
          MUST be defined.
        -->
        <xsl:element name="xsd:complexType">
          <!--
               [GTD1] 6.1.1
               All types MUST be named.
          -->
          <xsl:attribute name="name" 
                         select="concat(gu:col(.,$gu:names),'Type')"/>
          <!--document the construct-->
          <xsl:for-each select="$documentation">
            <xsl:element name="xsd:annotation">
              <xsl:element name="xsd:documentation">
                <xsl:apply-templates select="*" 
                                     mode="gu:documentation-replacement">
                  <xsl:with-param name="bie" tunnel="yes" select="$thisABIE"/>
                </xsl:apply-templates>
              </xsl:element>
            </xsl:element>
          </xsl:for-each>
          <!--the children of the construct-->
          <!--
       [MDC0] 3.2.1.1.4
       The sequence of the business information entities that is expressed in 
       the UBL model MUST be preserved in the schema.
       [CTD2] 6.1.3.1
       Every CCTS ABIE xsd:complexType definition content model MUST contain an
       xsd:sequence element containing the appropriate global element 
       declarations.
          -->
          <xsl:element name="xsd:sequence">
            <!--first, a document ABIE starts with the extension constructs,
                but not for the extension apex-->
            <xsl:if test="$schemaType=('DABIE','AABIE')">
              <!--﻿[ELD12] 3.4 
                The UBL Extensions element MUST be declared as the 
                first child of the document element with xsd:minOccurs="0".-->
              <xsl:apply-templates mode="gu:noRuntimeAnnotations" 
                    select="key('gu:files','EXT',$gu:config)/elements/node()"/>
            </xsl:if>
            <!--next the children of the ABIE-->
            <xsl:for-each select="key('gu:bie-by-abie',
                                      gu:col($thisABIE,$gu:names),
                                      $gu:gc)">
              <xsl:variable name="thisBIE" select="."/>
              <xsl:variable name="thisBIEtype"
                            select="gu:col(.,'ComponentType')"/>
              <xsl:variable name="thisBIEassociatedObjectClass"
                            select="gu:col(.,'AssociatedObjectClass')"/>
              <xsl:variable name="thisBIEname" select="gu:col(.,$gu:names)"/>
              <xsl:variable name="thisBIEisCommon"
                            select="$schemaType='CABIE' or
                                    gu:isInOtherLibrary($thisBIEname)"/>
              <xsl:element name="xsd:element">
                <xsl:attribute name="ref"
                               select="concat(
if( $thisBIEtype='BBIE' )
 then if( $thisBIEisCommon ) then $gu:cbbiePrefixColon else $thisCBCprefixColon
 else if( $thisBIEisCommon ) then $gu:cabiePrefixColon else $thisCACprefixColon
                                             ,gu:col(.,$gu:names))"/>
                <xsl:attribute name="minOccurs" select="gu:minMax(.,true())"/>
                <xsl:attribute name="maxOccurs" select="gu:minMax(.,false())"/>
                <!--document the construct-->
                <xsl:for-each select="$documentation">
                  <xsl:element name="xsd:annotation">
                    <xsl:element name="xsd:documentation">
                      <xsl:apply-templates select="*" 
                                           mode="gu:documentation-replacement">
                        <xsl:with-param name="bie" tunnel="yes" 
                                        select="$thisBIE"/>
                      </xsl:apply-templates>
                    </xsl:element>
                  </xsl:element>
                </xsl:for-each>
              </xsl:element>              
            </xsl:for-each>
            <!--the config file declaration may include some elements to add-->
            <xsl:copy-of select="$thisFile/elements[
                                   @abie=gu:col($thisABIE,$gu:names)]/node()"/>
          </xsl:element>
        </xsl:element>
      </xsl:for-each>
    </xsl:when>
    <xsl:when test="$schemaType=('CBBIE','SBBIE')">
      <xsl:call-template name="gu:sectionComment">
        <xsl:with-param name="section" 
          select="'Type Definitions',
                  'Basic Business Information Entity Type Definitions'"/>
      </xsl:call-template>
      <xsl:for-each-group select="$moduleBBIEs" group-by=".">
        <xsl:sort select="concat(.,'Type')"/>
        <xsl:if test="$schemaType = 'CBBIE' or
                      not( gu:isInOtherLibrary(.) )">
          <!--
             [CTD25] 6.1.3
             For every CCTS BBIE Property identified in the UBL model, a named 
             xsd:complexType MUST be defined.
          -->
          <xsl:element name="xsd:complexType">
            <!--
                 [GTD1] 6.1.1
                 All types MUST be named.
            -->
            <xsl:attribute name="name" select="concat(.,'Type')"/>
            <!--
       [CTD3] 6.1.3.2
       Every CCTS BBIE Property xsd:complexType definition content model MUST
       contain an xsd:simpleContent element.
            -->
            <xsl:element name="xsd:simpleContent">
              <!--
       [CTD4] 6.1.3.2
       Every CCTS BBIE Property xsd:complexType content model xsd:simpleContent
       element MUST consist of an xsd:extension element.
              -->
              <xsl:element name="xsd:extension">
                <!--
       [CTD26] 6.1.3.2
       Every CCTS BBIE Property xsd:complexType xsd:base attribute value MUST 
       be the UBL Unqualified Datatype.
                -->
                <xsl:attribute name="base">
                  <xsl:variable name="rawDataType"
                                select="gu:col(../..,'DataType')"/>
                  <!--in which library is this data item going to be found?-->
                  <xsl:choose>
                    <xsl:when test="contains($rawDataType, '_')
                                    and $qdt-as-cva='no'">
                  <xsl:value-of select="concat(key('gu:files','QDT',$gu:config)
                                                 /@prefix,':',
                                           translate($rawDataType,'_ .',''))"/>
                    </xsl:when>
                    <xsl:otherwise>
                      <xsl:variable name="dataType"
                                    select="if( contains( $rawDataType, '_' ) )
                                        then substring-after($rawDataType,'_')
                                        else $rawDataType"/>
                      <!--
       [CTD27] 6.1.3.2
       Every BBIE property with the representation term Code MUST be based on
       the UBL unqualified code data type.
                      -->
                  <xsl:value-of select="concat(key('gu:files','UDT',$gu:config)
                                               /@prefix,':',
                                               translate($dataType,' .',''))"/>
                    </xsl:otherwise>
                  </xsl:choose>
                </xsl:attribute>
              </xsl:element>
            </xsl:element>
          </xsl:element>
        </xsl:if>
      </xsl:for-each-group>
    </xsl:when>
  </xsl:choose>
</xsl:template>
  
<xs:template>
  <para>
    This template copies a documentation template, replacing the content
    of any element with text but no element children with the genericode
    column value of the name of that text.
  </para>
  <para>
    For example: <literal>&lt;description>Definition&lt;/description></literal>
    will produce an element named <literal>&lt;description></literal> with
    the content of the row's column named "Definition".
  </para>
  <xs:param name="bie">
    <para>The row in genericode for the BIE</para>
  </xs:param>
</xs:template>
<xsl:template match="*" mode="gu:documentation-replacement">
  <xsl:param name="bie" tunnel="yes" as="element(Row)"/>
  <xsl:choose>
    <xsl:when test="*">
      <!--preserve element structure down to the content of a leaf element-->
      <!--[MDC2] 3.2.1.1.3
    XML mixed content MUST NOT be used except where contained in an 
    xsd:documentation element.-->
      <xsl:copy>
        <xsl:copy-of select="@*"/>
        <xsl:apply-templates select="*" mode="gu:documentation-replacement"/>
      </xsl:copy>
    </xsl:when>
    <xsl:otherwise>
      <!--preserve leaf element if there is corresponding column content-->
      <xsl:if test="gu:col($bie,.)[normalize-space(.)]">
        <xsl:copy>
          <xsl:copy-of select="@*"/>
          <xsl:value-of select="gu:col($bie,.)"/>
        </xsl:copy>
      </xsl:if>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xs:function>
  <para>Determine the minimum or maximum occurrences from cardinality</para>
  <xs:param name="row">
    <para>The genericode row of the BIE</para>
  </xs:param>
  <xs:param name="minimumFlag">
    <para>Indication asking for the minimum cardinality</para>
  </xs:param>
</xs:function>
<xsl:function name="gu:minMax" as="xsd:string">
  <xsl:param name="row" as="element(Row)"/>
  <xsl:param name="minimumFlag" as="xsd:boolean"/>
  <xsl:for-each select="gu:col($row,'Cardinality')">
    <xsl:choose>
      <xsl:when test="$minimumFlag">
        <xsl:value-of select="substring(.,1,1)"/>
      </xsl:when>
      <xsl:when test="ends-with(.,'n')">unbounded</xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="substring(.,string-length(.),1)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:function>
  
<xs:template>
  <para>Prune XSD annotations while copying a schema fragment</para>
  <xs:param name="runtime">
    <para>
      An indication if the file being created is for a runtime directory
      as compared to a directory of fully-documented constructs.
    </para>
  </xs:param>
</xs:template>
<xsl:template match="xsd:annotation | text()" mode="gu:noRuntimeAnnotations">
  <xsl:param name="runtime" tunnel="yes" as="xsd:boolean"/>
  <xsl:if test="not($runtime)">
    <xsl:next-match/><!--go ahead and preserve this element; not in runtime-->
  </xsl:if>
</xsl:template>

<xs:template>
  <para>Preserve elements while copying a schema fragment</para>
</xs:template>
<xsl:template match="*" mode="gu:noRuntimeAnnotations">
  <xsl:copy>
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates mode="gu:noRuntimeAnnotations"/>
  </xsl:copy>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>CVA generator</xs:title>
</xs:doc>

<xs:template>
  <para>
    This is an identity transform replicating the skeleton CVA file but
    intercepting a reference to an NDR data type.  This template is the
    identity transform portion.
  </para>
</xs:template>
<xsl:template match="@*|node()" mode="gu:CVAWrite">
  <xsl:copy>
    <xsl:apply-templates mode="gu:CVAWrite" select="@*,node()"/>
  </xsl:copy>
</xsl:template>

<xs:template>
  <para>
    Translate <literal>NDRDataType=</literal> into a CVA
    <literal>address=</literal> attribute.
  </para>
</xs:template>
<xsl:template match="@NDRDataType" mode="gu:CVAWrite">
  <xsl:attribute name="address">
    <xsl:for-each-group select="key('gu:bbie-by-data-type',.,$gu:gc)"
                        group-by="gu:col(.,$gu:names)">
      <xsl:sort select="current-grouping-key()"/>
      <xsl:if test="position()>1"> | </xsl:if>
      <xsl:value-of select="concat($gu:cbbiePrefixColon,
                                   current-grouping-key())"/>
    </xsl:for-each-group>
  </xsl:attribute>
</xsl:template>
  
<!--========================================================================-->
<xs:doc>
  <xs:title>URI calculation functions</xs:title>
</xs:doc>
  
<xs:function>
  <para>
      Return a relative URI of the target config node relative to the source
      config node, perhaps using the runtime name of a directory.  This
      function merely feeds the looping function with an initial empty URI
      string.
  </para>
  <xs:param name="target">
    <para>The config element to which the URI points.</para>
  </xs:param>
  <xs:param name="source">
    <para>The config element from which the URI points.</para>
  </xs:param>
  <xs:param  name="runtime">
    <para>
      An indication that the runtime directory name is to be used in 
      preference to the directory name.
    </para>
  </xs:param>
</xs:function>
<xsl:function name="gu:relativeConfigURI" as="xsd:string">
  <xsl:param name="target" as="element()"/>
  <xsl:param name="source" as="element()"/>
  <xsl:param name="runtime" as="xsd:boolean"/>
 <xsl:sequence select="gu:relativeConfigURILoop($target,$source,$runtime,'')"/>
</xsl:function>

<xs:function>
  <para>
      Return a relative URI of the target config node relative to the source
      config node, perhaps using the runtime name of a directory, but based
      on a URI string in the process of being composed.
  </para>
  <xs:param name="target">
    <para>The config element to which the URI points.</para>
  </xs:param>
  <xs:param name="source">
    <para>The config element from which the URI points.</para>
  </xs:param>
  <xs:param  name="runtime">
    <para>
      An indication that the runtime directory name is to be used in
      preference to the directory name.
    </para>
  </xs:param>
  <xs:param name="URIinProgress">
    <para>The incremental building of a URI</para>
  </xs:param>
</xs:function>
<xsl:function name="gu:relativeConfigURILoop" as="xsd:string">
  <xsl:param name="target" as="element()"/>
  <xsl:param name="source" as="element()"/>
  <xsl:param name="runtime" as="xsd:boolean"/>
  <xsl:param name="URIinProgress" as="xsd:string"/>
  <xsl:choose>
    <!--first check to see if the target is a sibling-->
    <xsl:when test="$source/child::*[. is $target]">
      <xsl:value-of select="concat( $URIinProgress, $target/@name )"/>
    </xsl:when>
    <!--walk down the tree to see if the target is in a descendant directory-->
    <xsl:when test="$source/descendant::*[. is $target]">
      <xsl:sequence select="gu:relativeConfigURILoop( $target, 
        $source/child::dir[descendant::*[. is $target]],
        $runtime, concat( $URIinProgress, $source/
                            child::dir[descendant::*[. is $target]]/
    (if ($runtime and @runtime-name) then @runtime-name else @name), '/' ) )"/>
    </xsl:when>
    <!--move up to the directory to check for children-->
    <xsl:when test="$source/(self::file | self::files)">
      <xsl:sequence select="gu:relativeConfigURILoop
          ( $target, $source/.., $runtime, $URIinProgress )"/>
    </xsl:when>
    <!--move to next sibling configuration file without changing path-->
    <xsl:when test="$source/self::configuration">
      <xsl:sequence select="gu:relativeConfigURILoop
                        ( $target, $source/following-sibling::configuration[1], 
                          $runtime, $URIinProgress )"/>
    </xsl:when>
    <!--walk up along the directory tree to see if the target -->
    <xsl:otherwise>
      <xsl:sequence select="gu:relativeConfigURILoop
          ( $target, $source/.., $runtime, concat( $URIinProgress, '../' ) )"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:function>

<!--========================================================================-->
<xs:doc>
  <xs:title>Utility templates and functions</xs:title>
</xs:doc>
  
<xs:template>
  <para>Return any problems with the inputs and arguments</para>
  <!--
    not checked:  all pairs of sabie-prefix=/sbbie-prefix same or exclusive
  -->
</xs:template>
<xsl:template name="gu:validateInputs">
  <!--are the input parameters acceptable?-->
  <xsl:if test="not($qdt-as-cva=('yes','no'))">
    <xsl:text>Invalid yes/no value for parameter "qdt-as-cva": </xsl:text>
    <xsl:value-of select="$qdt-as-cva"/>
    <xsl:text>
</xsl:text>
  </xsl:if>
  
  <!--check all extension information is present-->
  <xsl:choose>
    <xsl:when test="boolean($common-gc-uri) != 
                  ($xabie-prefix or $aabie-prefix)">
      <xsl:text>When creating extension/addition constructs, both </xsl:text>
      <xsl:text>common-gc-uri and xabie-prefix parameters </xsl:text>
      <xsl:text>must be specified.
</xsl:text>
    </xsl:when>
    <xsl:when test="$xabie-prefix and not( key('gu:files','XABIE',$gu:config)
                                           [@prefix=$xabie-prefix] )">
      <xsl:text>Cannot find requested extension schema with prefix "</xsl:text>
      <xsl:value-of select="$xabie-prefix"/>
      <xsl:text>"
</xsl:text>
    </xsl:when>
    <xsl:when test="$aabie-prefix and not( key('gu:files','AABIE',$gu:config)
                                           [@prefix=$aabie-prefix] )">
     <xsl:text>Cannot find requested additional schema with prefix "</xsl:text>
      <xsl:value-of select="$aabie-prefix"/>
      <xsl:text>"
</xsl:text>
    </xsl:when>
  </xsl:choose>
  
  <!--check the input genericode files-->
  <xsl:for-each select="$gu:gc,$gu:gcOther">
    <xsl:choose>
      <xsl:when test="namespace-uri(/*) !=
                        'http://docs.oasis-open.org/codelist/ns/genericode/1.0/'
                      or local-name(/*) != 'CodeList'">
        <xsl:text>Expected {http://docs.oasis-open.org/codelist/ns/</xsl:text>
        <xsl:text>genericode/1.0/}CodeList as the document element of "</xsl:text>
        <xsl:value-of select="document-uri(/)"/>
        <xsl:text>", but found the document element: </xsl:text>
      <xsl:value-of select="concat('{',namespace-uri(/*),'}',local-name(/*))"/>
        <xsl:text>
</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name="abie-models" as="element()*">
          <xsl:for-each-group group-by="gu:col(.,'ModelName')"
            select="/*/SimpleCodeList/Row[gu:col(.,'ComponentType')='ABIE']">
              <model name="{current-grouping-key()}" 
                     abie-count="{count(current-group())}"/>
          </xsl:for-each-group>
        </xsl:variable>
        <xsl:if test="count($abie-models[@abie-count>1])>1">
          <xsl:text>Found in the genericode input too many common </xsl:text> 
          <xsl:text>library models with multiple ABIE constructs: </xsl:text>
          <xsl:value-of select="$abie-models[@abie-count>1]" separator=", "/>
          <xsl:text>
</xsl:text>
        </xsl:if>
      </xsl:otherwise>
    </xsl:choose>  
  </xsl:for-each>

  <!--check the configuration files-->
  <xsl:for-each select="$gu:configMain, $gu:configOther">
   <xsl:choose>
    <xsl:when test="not(/configuration)">
      <xsl:text>Expected {}configuration as the document element of </xsl:text>
      <xsl:text>the configuration file, but in "</xsl:text>
      <xsl:value-of select="document-uri(/)"/>
      <xsl:text>" found the document element: </xsl:text>
      <xsl:value-of select="concat('{',namespace-uri(/*),'}',local-name(/*))"/>
      <xsl:text>
</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <!--version= is a required attribute-->
      <xsl:if test="not(normalize-space(/*/@version))">
  <xsl:text>A version of the suite of schemas must be specified </xsl:text>
  <xsl:text>using version= on the configuration file document element.
</xsl:text>
      </xsl:if>
      <!--does every copyright have a position?-->
      <xsl:for-each select="//copyright">
        <xsl:if test="not(@position=('start','end'))">
          <xsl:text>Every copyright statement must have </xsl:text>
          <xsl:text>position="start" or position="end": </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
</xsl:text>
        </xsl:if>
      </xsl:for-each>
      <!--does every file element have reqired attributes?-->
      <xsl:for-each select="//(file|files)">
        <xsl:choose>
          <xsl:when test="@type='CVA' and not(normalize-space(@name)) and
                          not(normalize-space(@skeleton))">
            <xsl:text>Each of type= and name= and skeleton= must be </xsl:text>
            <xsl:text>specified: </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
</xsl:text>
          </xsl:when>
          <xsl:when test="not(normalize-space(@type)) or 
                          not(normalize-space(@name)) or
                          (@type != 'CVA' and not(normalize-space(@namespace)))">
            <xsl:text>Each of type=, name= and namespace= must be </xsl:text>
            <xsl:text>specified: </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
</xsl:text>
          </xsl:when>
          <xsl:when test="not(@type=('CABIE','CBBIE','QDT','UDT','EXT','DABIE',
                                     'CVA','XABIE','AABIE','SABIE','SBBIE'))">
            <xsl:text>Invalid type attribute: </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
</xsl:text>
          </xsl:when>
          <xsl:when test="@abie and (not(self::file) or 
                                     not(@type=('DABIE','AABIE')))">
            <xsl:text>abie= can only be specified for a single </xsl:text>
            <xsl:text>file with type 'DABIE' or 'AABIE': </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
    </xsl:text>
          </xsl:when>
          <xsl:when test="@type=('DABIE','AABIE','XABIE') and
                          count($gu:documentABIEs)=0">
     <xsl:text>No document-level ABIEs supplied to satisfy type='</xsl:text>
            <xsl:value-of select="@type"/>
            <xsl:text>': </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
</xsl:text>
          </xsl:when>
          <xsl:when test="( @type='DABIE' or
                            ( @type='AABIE' and @prefix=$aabie-prefix ) or
                            ( @type='XABIE' and @prefix=$xabie-prefix ) ) and 
                          self::file and 
                          not(@abie) and count($gu:documentABIEs)>1">
     <xsl:text>abie= must be specified for a single file when type='</xsl:text>
            <xsl:value-of select="@type"/>
          <xsl:text>' and there exists more than one document ABIE: </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
</xsl:text>
          </xsl:when>
          <xsl:when test="self::files and 
                          not(@type=( 'DABIE','XABIE','AABIE'))">
   <xsl:text>&lt;files> must have a type of DABIE, XABIE or AABIE': </xsl:text>
            <xsl:value-of select="gu:reportElement(.)"/>
            <xsl:text>
</xsl:text>
          </xsl:when>
        </xsl:choose>
        <!--additional tests for CVA files; the skeleton must exist-->
        <xsl:if test="@type='CVA'">
          <xsl:variable name="skeleton" select="document(@skeleton-uri)"/>
          <xsl:if test="not($skeleton)">
        <xsl:text>Unable to open specified input CVA skeleton file "</xsl:text>
              <xsl:value-of select="@skeleton-uri"/>
              <xsl:text>" relative to configuration file: </xsl:text>
              <xsl:value-of select="base-uri(@skeleton-uri)"/>
            <xsl:text>
</xsl:text>
          </xsl:if>
          <xsl:for-each select="$skeleton">
            <xsl:if test="local-name(/*) != 'ContextValueAssociation' or
                          namespace-uri(/*) !=
        'http://docs.oasis-open.org/codelist/ns/ContextValueAssociation/1.0/'">
      <xsl:text>Expected {http://docs.oasis-open.org/codelist/ns/</xsl:text>
      <xsl:text>ContextValueAssociation/1.0/}ContextValueAssociation</xsl:text>
      <xsl:text> as the document element of </xsl:text>
      <xsl:text>the input file "</xsl:text>
      <xsl:value-of select="document-uri(/)"/>
      <xsl:text>", but found the document element: </xsl:text>
      <xsl:value-of select="concat('{',namespace-uri(/*),'}',local-name(/*))"/>
      <xsl:text>
</xsl:text>
            </xsl:if>
          </xsl:for-each>
        </xsl:if>
      </xsl:for-each>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:for-each>
</xsl:template>
  
<xs:function>
  <para>Report a start tag for error messages.</para>
  <xs:param name="this">
    <para>The element being reported</para>
  </xs:param>
</xs:function>
<xsl:function name="gu:reportElement" as="xsd:string">
  <xsl:param name="this" as="element()"/>
  <xsl:value-of>
    <xsl:for-each select="$this/ancestor-or-self::*">
      <xsl:text/>/<xsl:value-of select="name(.)"/>
      <xsl:if test="parent::*">[<xsl:number/>]</xsl:if>
    </xsl:for-each>
    <xsl:text>: </xsl:text>
    <xsl:for-each select="$this">
      <xsl:text/>&lt;<xsl:value-of select="name(.)"/>
      <xsl:for-each select="@*">
        <xsl:value-of select="concat(' ',name(.),'=&#34;',.,'&#34;')"/>
      </xsl:for-each>
      <xsl:text>></xsl:text>
    </xsl:for-each>
  </xsl:value-of>
</xsl:function>

<xs:template>
  <para>
    Format section comments consistently.  All supplied sections are
    emitted on adjacent lines.  Use separate calls to separate comments
    with a blank line.
  </para>
  <xs:param name="section">
    <para>Used to identify the section.</para>
  </xs:param>
  <xs:param name="runtime">
    <para>Indication in runtime mode so suppress section comments</para>
  </xs:param>
</xs:template>
<xsl:template name="gu:sectionComment">
  <xsl:param name="section" as="xsd:string+"/>
  <xsl:param name="runtime" tunnel="yes" as="xsd:boolean"/>
  <xsl:if test="not($runtime)">
    <xsl:text>
   </xsl:text>
    <xsl:for-each select="$section">
      <xsl:comment> ===== <xsl:value-of select="."/> ===== </xsl:comment>
      <xsl:text>
   </xsl:text>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:function>
  <para>
    Return true() if the construct defined by the passed row is a construct
    found in the $gu:gcOther input.
  </para>
  <xs:param name="thisBIE">
    <para>The BIE name.</para>
  </xs:param>
</xs:function>
<xsl:function name="gu:isInOtherLibrary" as="xsd:boolean">
  <xsl:param name="thisBIE" as="element(SimpleValue)"/>
  <xsl:variable name="row" select="$thisBIE/../.."/>
  <xsl:choose>
    <xsl:when test="gu:col($row,'ComponentType')='BBIE'">
      <!--if the construct is a BBIE, simply check all of the same name-->
      <xsl:sequence select="boolean($gu:gcOther/key('gu:bie-by-type','BBIE',.)
                                            [gu:col(.,$gu:names)=$thisBIE]) "/>
    </xsl:when>
    <xsl:when test="gu:col($row,'ComponentType')='ASBIE'">
      <!--if the construct is a ASBIE, simply check all of the same name-->
      <xsl:sequence select="boolean($gu:gcOther/key('gu:bie-by-type','ASBIE',.)
                                            [gu:col(.,$gu:names)=$thisBIE]) "/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:sequence select="boolean($gu:gcOther/key('gu:bie-by-type','ABIE',.)
                            [gu:col(.,'ModelName')=$gu:otherCommonLibraryModel]
                            [gu:col(.,$gu:names)=$thisBIE])"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:function>
</xsl:stylesheet>
