<?xml version="1.0" encoding="US-ASCII"?>
<?xml-stylesheet type="text/xsl" href="../../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
                xmlns:gu="urn:X-gc2ublndr"
                exclude-result-prefixes="xs gu xsd"
                version="2.0">
<xsl:import href="Crane-commonndr.xsl"/>

<xs:doc info="$Id: Crane-ublndrChecker.xsl,v 1.14 2011/11/27 19:51:35 admin Exp $"
        filename="Crane-ublndrChecker.xsl" vocabulary="DocBook">
  <xs:title>UBL NDR Checker</xs:title>
  <para>
    This reports all consistency checks on the completeness of the
    combination of a Crane-gc2ublndr.xsl configuration input file and a UBL NDR
    model as serialized in a genericode file.  When the inputs pass these
    consistency checks, then
    <ulink url="readme-Crane-gc2ublndr.html"
                                    ><literal>Crane-gc2ublndr</literal></ulink>
    will produce a suite of schemas following the UBL Naming and Design Rules.
  </para>
  <para>
    When a common genericode file is not supplied as an optional argument, it
    is assumed the input genericode file is standalone and is checked to be
    entirely self-consistent.
  </para>
  <para>
    When a common genericode file is supplied as an optional argument, it is
    assumed the input genericode file is for extensions.  The common genericode
    file is not checked for consistency, so this should have been done on the
    common genericode file as a standalone check.  A dangling reference in
    the input file to a construct in the common file is not regarded as an
    error.
  </para>
  <para>
    Note that not all rules are modeling rules, and only modeling rules can be
    checked with this stylesheet.  Every rule is listed, including
    non-modeling rules, but the non-modeling rules are simply no-operation.
  </para>
  <para>
    Some of the rules cited are suffixed with a letter.  This is a rule that
    is not in the NDR but is closely enough related to an existing NDR to cite
    the existing NDR.  The new rule should be considered for inclusion in a
    future NDR.
  </para>
  <section>
    <title>Rules that are not checked</title>
    <para>
      The following rules are not considered checkable or relevant to the 
      input files, ordered by their presence in the NDR documentation:
    </para>
    <itemizedlist>
       <listitem><emphasis role="bold">[NMS3]</emphasis> 3.5.1
       UBL namespaces MUST only contain UBL developed schema modules.</listitem>
       <listitem><emphasis role="bold">[NMS4]</emphasis> 3.5.2
       The namespace names for UBL schemas holding committee draft status MUST
       be of the form urn:oasis:names:tc:ubl:schema:&lt;subtype>:&lt;document-id></listitem>
       <listitem><emphasis role="bold">[NMS5]</emphasis> 3.5.2
       The namespace names for UBL schemas holding OASIS Standard status MUST 
       be of the form 
       urn:oasis:names:specification:ubl:schema:&lt;subtype>:&lt;document-id></listitem>
       <listitem><emphasis role="bold">[NMS6]</emphasis> 3.5.4
       UBL published namespaces MUST never be changed.</listitem>
       <listitem><emphasis role="bold">[NMS19]</emphasis> 3.7.4
       The CCTS Core Component Type schema module must be represented by the 
       namespace prefix "ccts-cct".</listitem>
       <listitem><emphasis role="bold">[VER5]</emphasis> 3.6.1
       For UBL minor version changes, the namespace name MUST not change.</listitem>
       <listitem><emphasis role="bold">[VER7]</emphasis> 3.6.2
       Every UBL schema module minor version number MUST be a sequentially 
       assigned, nonnegative integer.</listitem>
       <listitem><emphasis role="bold">[SSM1]</emphasis> 3.7
       UBL schema expressions MAY be split into multiple schema modules.</listitem>
       <listitem><emphasis role="bold">[SSM2]</emphasis> 3.7.1
       A schema in one UBL namespace that is dependent upon type definitions 
       or element declarations in another schema namespace MUST only import 
       that schema.</listitem>
       <listitem><emphasis role="bold">[SSM3]</emphasis> 3.7.1
       A schema in one UBL namespace that is dependent upon type definitions 
       or element declarations defined in another schema namespace MUST NOT 
       import the internal schema modules of that schema.</listitem>
       <listitem><emphasis role="bold">[SSM6]</emphasis> 3.7.3
       All UBL internal schema modules MUST be in the same namespace as their
       corresponding document schema.</listitem>
       <listitem><emphasis role="bold">[SSM7]</emphasis> 3.7.3
       Each UBL internal schema module MUST be named 
       &lt;ParentSchemaModuleName>&lt;InternalSchemaModuleFunction></listitem>
       <listitem><emphasis role="bold">[SSM8]</emphasis> 3.7.4
       UBL schema modules MAY be created for reusable components.</listitem>
       <listitem><emphasis role="bold">[SSM22]</emphasis> 3.7.4
       The UBL Qualified Datatypes schema module MUST import the UBL 
       Unqualified Datatypes schema module.</listitem>
       <listitem><emphasis role="bold">[NMS20]</emphasis> 3.7.4.4
       The UBL Unqualified Datatypes schema module namespace MUST be 
       represented by the prefix "udt" when referenced in other schemas.</listitem>
       <listitem><emphasis role="bold">[SSM18]</emphasis> 3.7.4.5
       A schema module without any declarations must exist.</listitem>
        <listitem>
       <emphasis role="bold">[DOC1]</emphasis> 4.2
       The xsd:documentation element for every data type MUST contain a set of
       annotations in the following order (as defined in CCTS Section 7):
        <itemizedlist>
          <listitem>DictionaryEntryName (mandatory)</listitem>
          <listitem>Version (mandatory)</listitem>
          <listitem>Definition (mandatory)</listitem>
          <listitem>RepresentationTerm (mandatory)</listitem>
          <listitem>QualifierTerm(s) (mandatory, where used)</listitem>
          <listitem>UniqueIdentifier (mandatory)</listitem>
          <listitem>Usage Rule(s) (optional)</listitem>
          <listitem>Content Component Restriction (optional)</listitem>
        </itemizedlist>
       </listitem>
       <listitem><emphasis role="bold">[DOC2]</emphasis> 4.2
       A datatype definition MAY contain one or more Content Component 
       Restrictions to provide additional information on the relationship
       between the datatype and its corresponding Core Component Type. If used,
       the Content Component Restrictions MUST contain a set of annotations in
       the following order:
        <itemizedlist><listitem>RestrictionType (mandatory): Defines the type of format restriction
        that applies to the Content Component.</listitem>
        <listitem>RestrictionValue (mandatory): The actual value of the format 
        restriction that applies to the Content Component.</listitem>
        <listitem>ExpressionType (optional): Defines the type of the regular expression
        of the restriction value.</listitem></itemizedlist></listitem>
       <listitem><emphasis role="bold">[DOC3]</emphasis> 4.2
       A datatype definition MAY contain one or more Supplementary Component 
       Restrictions to provide additional information on the relationship 
       between the datatype and its corresponding Core Component Type. If used,
       the Supplementary Component Restrictions MUST contain a set of
       annotations in the following order:
        <itemizedlist><listitem>SupplementaryComponentName (mandatory): Identifies the Supplementary 
        Component to which the restriction applies.</listitem>
        <listitem>RestrictionValue (mandatory, repetitive): The actual value(s) that is
        (are) valid for the Supplementary Component.</listitem></itemizedlist></listitem>
       <listitem><emphasis role="bold">[DOC8]</emphasis> 4.2
       The xsd:documentation element for every Supplementary Component
       attribute declaration MUST contain a set of annotations in the following
       order:
      <itemizedlist>
        <listitem>Name (mandatory): Name in the Registry of a Supplementary Component
        of a Core Component Type.</listitem>
        <listitem>Definition (mandatory): An explanation of the meaning of a 
        Supplementary Component and its relevance for the related Core 
        Component Type.</listitem>
        <listitem>Primitive type (mandatory): The PrimitiveType to be used for the
        representation of the value of a Supplementary Component.</listitem>
        <listitem>Possible Value(s) (optional): Possible values of Supplementary
        Components.</listitem>
      </itemizedlist>
       </listitem>
       <listitem><emphasis role="bold">[DOC9]</emphasis> 4.2
       The xsd:documentation element for every Supplementary Component
       attribute declaration containing restrictions MUST include the following
       additional information appended to the information required by DOC8:
        - <itemizedlist><listitem>Restriction Value(s) (mandatory): The actual value(s) that is (are)
        valid for the Supplementary Component.</listitem></itemizedlist></listitem>
       <listitem><emphasis role="bold">[GNR1]</emphasis> 5.1
       UBL XML element and type names MUST be in the English language, using
       the primary English spellings provided in the Oxford English Dictionary.</listitem>
       <listitem><emphasis role="bold">[GNR7]</emphasis> 5.1
       UBL XML element and type names MUST be in singular form unless the 
       concept itself is plural.</listitem>
       <listitem><emphasis role="bold">[GNR10]</emphasis> 5.1
       Acronyms and abbreviations at the beginning of an attribute name MUST 
       appear in all lower case. Acronyms and abbreviations elsewhere in an
       attribute name MUST appear in upper case.</listitem>
       <listitem><emphasis role="bold">[GNR4]</emphasis> 5.1
       UBL XML element names and simple and complex type names MUST NOT use 
       acronyms, abbreviations, or other word truncations, except those in the
       list of exceptions maintained and published by the UBL TC.</listitem>
       <listitem><emphasis role="bold">[GNR6]</emphasis> 5.1
       The acronyms and abbreviations listed in the UBL-approved list MUST 
       always be used in place of the word or phrase they represent.</listitem>
       <listitem><emphasis role="bold">[GNR9]</emphasis> 5.1
       The lowerCamelCase (LCC) convention MUST be used for naming attributes.</listitem>
       <listitem><emphasis role="bold">[GNR11]</emphasis> 5.1
       Acronyms and abbreviations MUST appear in all upper case for all
       element and type names.</listitem>
       <listitem><emphasis role="bold">[GTD2]</emphasis> 6.1.1
       The predefined XML schema type xsd:anyType MUST NOT be used.</listitem>
       <listitem><emphasis role="bold">[CDL1]</emphasis> 7
       All UBL codes MUST be part of a UBL or externally maintained code list.</listitem>
       <listitem><emphasis role="bold">[CDL2]</emphasis> 7
       The UBL Library SHOULD identify and use external standardized code lists
       rather than develop its own UBL-native code lists.</listitem>
       <listitem><emphasis role="bold">[CDL3]</emphasis> 7
       The UBL Library MAY design and use an internally maintained code list
       where an existing externally maintained code list needs to be extended,
       or where no suitable externally maintained code list exists.</listitem>
       <listitem><emphasis role="bold">[GXS3]</emphasis> 8.1
       Built-in xsd:simpleTypes SHOULD be used wherever possible.</listitem>
       <listitem><emphasis role="bold">[GXS5]</emphasis> 8.3
       The xsd:substitutionGroup feature MUST NOT be used.</listitem>
       <listitem><emphasis role="bold">[GXS6]</emphasis> 8.4
       The xsd:final attribute MUST be used to control extensions where there
       is a desire to prohibit further extensions.</listitem>
       <listitem><emphasis role="bold">[GXS7]</emphasis> 8.5
       xsd:notation MUST NOT be used.</listitem>
       <listitem><emphasis role="bold">[GXS8]</emphasis> 8.6
       xsd:all MUST NOT be used.</listitem>
       <listitem><emphasis role="bold">[GXS9]</emphasis> 8.7
       The xsd:choice element SHOULD NOT be used where customization and
       extensibility are a concern.</listitem>
       <listitem><emphasis role="bold">[GXS10]</emphasis> 8.8
       xsd:include can only be used when the including schema is in the same
       namespace as the included schema.</listitem>
       <listitem><emphasis role="bold">[GXS11]</emphasis> 8.9
       The xsd:union technique MUST NOT be used except for code lists.</listitem>
       <listitem><emphasis role="bold">[GXS12]</emphasis> 8.10
       UBL schemas SHOULD NOT use xsd:appinfo. If used, xsd:appinfo MUST be
       used only to convey non-normative information.</listitem>
       <listitem><emphasis role="bold">[GXS16]</emphasis> 8.12
       The built in xsd:nillable attribute MUST NOT be used for any UBL 
       declared element.</listitem>
       <listitem><emphasis role="bold">[GXS14]</emphasis> 8.13
       xsd:any MUST NOT be used except within the ExtensionContentType type
       definition, and with xsd:processContents= "skip" for non-UBL namespaces.</listitem>
       <listitem><emphasis role="bold">[GXS13]</emphasis> 8.14
       Complex type extension or restriction MAY be used where appropriate.</listitem>
    </itemizedlist>
    <para>
      The following rules are addressed in the writing of the 
      Crane-gc2ublndr.xsl code itself and cannot be tested on the inputs:
    </para>
    <itemizedlist>
       <listitem><emphasis role="bold">[RED2]</emphasis> 3.1.2
       The root element MUST be the only global element declared in the 
       document schema.</listitem>
       <listitem><emphasis role="bold">[MDC2]</emphasis> 3.2.1.1.3
       XML mixed content MUST NOT be used except where contained in an 
       xsd:documentation element.</listitem>
       <listitem><emphasis role="bold">[MDC0]</emphasis> 3.2.1.1.4
       The sequence of the business information entities that is expressed in 
       the UBL model MUST be preserved in the schema.</listitem>
       <listitem><emphasis role="bold">[ELD2]</emphasis> 3.3.1
       All element declarations MUST be global.</listitem>
       <listitem><emphasis role="bold">[ELD12]</emphasis> 3.4 
       The UBL Extensions element MUST be declared as the 
       first child of the document element with xsd:minOccurs="0".</listitem>
       <listitem><emphasis role="bold">[MDC2]</emphasis> 3.2.1.1.3
       XML mixed content MUST NOT be used except where contained in an 
       xsd:documentation element.</listitem>
       <listitem><emphasis role="bold">[CTN1]</emphasis> 5.2.1
       A UBL xsd:complexType name based on a CCTS ABIE MUST be the CCTS 
       Dictionary Entry Name with the separators removed and with the "Details"
       suffix replaced with "Type".</listitem>
       <listitem><emphasis role="bold">[CTN2]</emphasis> 5.2.2
       A UBL xsd:complexType name based on a CCTS BBIE Property MUST be the 
       CCTS Dictionary Entry Name shared Property Term and its qualifiers and
       the Representation Term of the BBIE with the separators removed and with
       the "Type" suffix appended after the Representation Term.</listitem>
       <listitem><emphasis role="bold">[ELN1]</emphasis> 5.3.1
       A UBL global element name based on a CCTS ABIE MUST be the same as the 
       name of the corresponding xsd:complexType to which it is bound, with the
       word "Type" removed.</listitem>
       <listitem><emphasis role="bold">[ELN2]</emphasis> 5.3.2
       A UBL global element name based on a CCTS BBIE Property MUST be the same
       as the name of the corresponding xsd:complexType to which it is bound, 
       with the word "Type" removed.</listitem>
       <listitem><emphasis role="bold">[GTD1]</emphasis> 6.1.1
       All types MUST be named.</listitem>
       <listitem><emphasis role="bold">[CTD1]</emphasis> 6.1.3
       For every class identified in the UBL model, a named xsd:complexType 
       MUST be defined.</listitem>
       <listitem><emphasis role="bold">[CTD25]</emphasis> 6.1.3
       For every CCTS BBIE Property identified in the UBL model, a named 
       xsd:complexType MUST be defined.</listitem>
       <listitem><emphasis role="bold">[CTD2]</emphasis> 6.1.3.1
       Every CCTS ABIE xsd:complexType definition content model MUST contain an
       xsd:sequence element containing the appropriate global element 
       declarations.</listitem>
       <listitem><emphasis role="bold">[CTD3]</emphasis> 6.1.3.2
       Every CCTS BBIE Property xsd:complexType definition content model MUST
       contain an xsd:simpleContent element.</listitem>
       <listitem><emphasis role="bold">[CTD4]</emphasis> 6.1.3.2
       Every CCTS BBIE Property xsd:complexType content model xsd:simpleContent
       element MUST consist of an xsd:extension element.</listitem>
       <listitem><emphasis role="bold">[CTD26]</emphasis> 6.1.3.2
       Every CCTS BBIE Property xsd:complexType xsd:base attribute value MUST 
       be the UBL Unqualified Datatype.</listitem>
       <listitem><emphasis role="bold">[CTD27]</emphasis> 6.1.3.2
       Every BBIE property with the representation term Code MUST be based on
       the UBL unqualified code data type.</listitem>
       <listitem><emphasis role="bold">[ELD3]</emphasis> 6.2.1
       For every class and property identified in the UBL model, a global 
       element bound to the corresponding xsd:complexType MUST be declared.</listitem>
       <listitem><emphasis role="bold">[ELD4]</emphasis> 6.2.2
       When a CCTS ASBIE is unqualified, it is bound via reference to the 
       global CCTS ABIE element with which it is associated.</listitem>
       <listitem><emphasis role="bold">[ELD11]</emphasis> 6.2.2
       When a CCTS ASBIE is qualified, a new element MUST be declared and bound
       to the xsd:complexType of its associated CCTS ABIE.</listitem>
       <listitem><emphasis role="bold">[GXS4]</emphasis> 8.2
       All XSD constructs in UBL schema and schema modules MUST contain the
       following namespace declaration on the xsd:schema element:
        xmlns:xsd="http://www.w3.org/2001/XMLSchema"</listitem>
       <listitem><emphasis role="bold">[GXS15]</emphasis> 8.11
       Each xsd:schemaLocation attribute declaration MUST contain a 
       system-resolvable URL, which at the time of release from OASIS shall be
       a relative URL referencing the location of the schema or schema module
       in the release package.</listitem>
    </itemizedlist>
  </section>
</xs:doc>

<xs:output>
  <para>This is a simple text report</para>
</xs:output>
<xsl:output method="text"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Input parameters</xs:title>
</xs:doc>

<xs:param ignore-ns="yes">
  <para>Bias the row count assuming input from a spreadsheet.</para>
</xs:param>
<xsl:param name="header-row-count" select="1" as="xsd:integer"/>

<xs:param ignore-ns="yes">
  <para>Switch to ignore rules specific to UBL project.</para>
</xs:param>
<xsl:param name="ubl-specific" select="'yes'" as="xsd:string"/>

<xs:param ignore-ns="yes">
  <para>
    Indicate this is checking the model of an extension rather than the model
    of a document.
  </para>
</xs:param>
<xsl:param name="extension" select="'no'" as="xsd:string"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Global variables and key tables</xs:title>
</xs:doc>

<xs:variable>
  <para>
    Build from the both configuration files a regular expression of
    abbreviations to use for element name validation.  
    An element's name uses UpperCamelCase conventions except
    for the abbreviations.
  </para>
</xs:variable>
<xsl:variable name="gu:UCC-abbrev" as="xsd:string"
    select="string-join(('^(([A-Z][a-z]+)',
                       for $each in $gu:config/*/*/abbreviations/abbreviation
                           return concat( '|', $each ),
                         ')+$'),'')"/>

<xs:key>
  <para>
    Index the genericode file for all business information entities by 
    dictionary entry name
  </para>
</xs:key>
<xsl:key name="gu:bie-by-den" match="/*/SimpleCodeList/Row"
         use="gu:col(.,'DictionaryEntryName')"/>

<xs:variable>
  <para>
    Build the list of available types from both configuration files.
  </para>
</xs:variable>
<xsl:variable name="gu:CCTS-types" as="xsd:string*"
              select="$gu:config/*/*/types/type"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Check configuration file and every row of the model</xs:title>
</xs:doc>

<xs:template>
  <para>
    Visit the configuration file and then every row of the input, acting on
    each of the rules in the order the rules are documented in the NDR
    documentation.
  </para>
</xs:template>
<xsl:template match="/">
  <!--first check show-stopping arguments-->
  <xsl:if test="not( $ubl-specific=('yes','no') )">
    <xsl:message terminate="yes" 
                 select="'Unexpected ubl-specific= value:',$ubl-specific"/>
  </xsl:if>
  <!--next analyze the inputs, simultaneously reporting through the
      message facility, yet keeping track of the presence of messages-->
  <xsl:variable name="allMessages" as="xsd:string*">
    <!--are there any problems with the main configuration file-->
    <xsl:for-each select="$gu:config/combination/configuration[1]">
      <xsl:variable name="message" as="xsd:string*">
        <xsl:call-template name="gu:rule-NMS2"/><!--3.5.1-->
        <xsl:call-template name="gu:rule-VER2-VER4-VER6"/><!--3.6.1-->
        <xsl:call-template name="gu:rule-VER12-VER14"/><!--3.6.2-->
        <xsl:call-template name="gu:rule-SSM21"/><!--3.7.4-->
        <xsl:call-template name="gu:rule-NMS18"/><!--3.7.4-->
        <xsl:call-template name="gu:rule-SSM9"/><!--3.7.4.1-->
        <xsl:call-template name="gu:rule-SSM10"/><!--3.7.4.1-->
        <xsl:call-template name="gu:rule-NMS8"/><!--3.7.4.1-->
        <xsl:call-template name="gu:rule-SSM11"/><!--3.7.4.2-->
        <xsl:call-template name="gu:rule-SSM12"/><!--3.7.4.2-->
        <xsl:call-template name="gu:rule-NMS10"/><!--3.7.4.2-->
        <xsl:call-template name="gu:rule-NMS16"/><!--3.7.4.5-->
        <xsl:call-template name="gu:rule-NMS16b"/><!--3.7.4.5-->
        <xsl:call-template name="gu:rule-SSM19"/><!--3.7.4.5-->
        <xsl:call-template name="gu:rule-SSM19b"/><!--3.7.4.5-->
        <xsl:call-template name="gu:rule-GXS2"/><!--4.1-->
        <xsl:call-template name="gu:rule-DOC4"/><!--4.2-->
        <xsl:call-template name="gu:rule-DOC5"/><!--4.2-->
        <xsl:call-template name="gu:rule-DOC6"/><!--4.2-->
      </xsl:variable>
      <xsl:if test="exists($message[string(.)])">
        <!--report the messages pertaining to the file-->
        <xsl:text>Configuration file messages:</xsl:text>      
        <xsl:for-each select="$message[string(.)]">
          <xsl:value-of select="' ',."/>
        </xsl:for-each>
      </xsl:if>
    </xsl:for-each>
    <!--are there any problems with any of the rows-->
    <xsl:for-each-group select="/*/SimpleCodeList/Row"
                        group-by="gu:col(.,'ModelName')">
     <xsl:for-each select="current-group()">
      <!--at a minimum, at least all the fields need to be present-->
      <xsl:variable name="minimum" as="xsd:string*">
        <xsl:call-template name="gu:checkMinimumValues"/>
      </xsl:variable>
      <!--check all of the other properties of the row-->
      <xsl:variable name="message" as="xsd:string*">
        <xsl:call-template name="gu:checkRequiredValues"/>
        <xsl:call-template name="gu:rule-NMC1"/><!--3.2.1-->
        <xsl:call-template name="gu:rule-MDC1"/><!--3.2.1.1.2-->
        <xsl:call-template name="gu:rule-ELD13x"/><!--3.4-->
        <xsl:call-template name="gu:rule-ELD14x"/><!--3.4-->
        <xsl:call-template name="gu:rule-ELD14x2"/><!--3.4-->
        <xsl:call-template name="gu:rule-ELD14x3"/><!--3.4-->
        <xsl:call-template name="gu:rule-GNR2"/><!--5.1-->
        <xsl:call-template name="gu:rule-GNR3"/><!--5.1-->
        <xsl:call-template name="gu:rule-GNR8"/><!--5.1-->
        <xsl:call-template name="gu:rule-CTN2"/><!--5.2.2-->
        <xsl:call-template name="gu:rule-CTN6"/><!--5.2.2-->
        <xsl:call-template name="gu:rule-CTN7"/><!--5.2.2-->
        <xsl:call-template name="gu:rule-CTN8"/><!--5.2.2-->
        <xsl:call-template name="gu:rule-ELN3"/><!--5.3.3-->
        <xsl:call-template name="gu:rule-ELD7"/><!--6.3-->
      </xsl:variable>
      <!--when there is a problem, report it-->
      <xsl:if test="exists($minimum[string(.)]) or 
                    exists($message[string(.)])">
        <!--report the messages pertaining to the row-->
        <xsl:value-of select="concat(current-grouping-key(),' Row ',
                                     position()+$header-row-count,' (',
                                     gu:col(.,'DictionaryEntryName'),'):')"/>
        <!--don't bother with the messages if minimum hasn't been met-->
        <xsl:for-each select="if( exists($minimum[string(.)]) ) 
                              then $minimum[string(.)] 
                              else $message[string(.)]">
          <xsl:value-of select="' ',."/>
        </xsl:for-each>
      </xsl:if>
     </xsl:for-each>
    </xsl:for-each-group>
  </xsl:variable>
 <!--trigger a fatal error when there are any messages at all-->
 <xsl:if test="exists($allMessages[string(.)])">
   <xsl:for-each select="$allMessages[string(.)]">
     <xsl:message select="."/>
   </xsl:for-each>
   <xsl:message terminate="yes">Problems detected</xsl:message>
 </xsl:if>
 <!--signal no problems-->
 <xsl:text>No problems detected.
</xsl:text>
</xsl:template>

<xs:template>
  <para>
       Check the row for having all of the required values based on type.
  </para>
</xs:template>
<xsl:template name="gu:checkMinimumValues" as="xsd:string?">
  <xsl:choose>
    <xsl:when test="not(gu:col(.,'ComponentType')[string(.)])">
      <xsl:text>Component type is absent</xsl:text>
    </xsl:when>
    <xsl:when test="not( gu:col(.,'ComponentType')=('ABIE','BBIE','ASBIE') )">
      <xsl:value-of>
        <xsl:text>Component type is not valid: </xsl:text>
        <xsl:value-of select="gu:col(.,'ComponentType')"/>
      </xsl:value-of>
    </xsl:when>
  </xsl:choose>
</xsl:template>

  <xs:template>
  <para>
       Check the row for having all of the required values based on type.
  </para>
</xs:template>
<xsl:template name="gu:checkRequiredValues" as="xsd:string*">
  <xsl:if test="not(gu:col(.,$gu:names)[string(.)])">
    <xsl:text>Missing the element name</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'DictionaryEntryName')[string(.)])">
    <xsl:text>Missing the dictionary entry name</xsl:text>
  </xsl:if>
  <xsl:if test="gu:col(.,'ObjectClassQualifier')[string(.)] and $ubl-specific">
    <xsl:text>UBL projects do not use object class qualifiers</xsl:text>
  </xsl:if>
  <xsl:if test="gu:col(.,'AssociatedObjectClassQualifier')[string(.)] and 
                $ubl-specific">
    <xsl:text
         >UBL projects do not use associated object class qualifiers</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'ObjectClass')[string(.)])">
    <xsl:text>Missing the object class</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'Definition')[string(.)])">
    <xsl:text>Missing the definition</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'PropertyTerm')[string(.)]) and
                gu:col(.,'ComponentType')=('BBIE','ASBIE')">
    <xsl:text>Missing the property term</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'RepresentationTerm')[string(.)]) and
                gu:col(.,'ComponentType')=('BBIE','ASBIE')">
    <xsl:text>Missing the representation term</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'DataType')[string(.)]) and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:text>Missing the data type</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'AssociatedObjectClass')[string(.)]) and
                gu:col(.,'ComponentType')='ASBIE'">
    <xsl:text>Missing the associated object class</xsl:text>
  </xsl:if>
  <xsl:if test="not(gu:col(.,'Cardinality')[string(.)]) and
                gu:col(.,'ComponentType')=('BBIE','ASBIE')">
    <xsl:text>Missing the cardinality</xsl:text>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[CTN] complex type naming rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[CTN2]</emphasis> 5.2.2
       A UBL xsd:complexType name based on a CCTS BBIE Property MUST be the 
       CCTS Dictionary Entry Name shared Property Term and its qualifiers and
       the Representation Term of the BBIE with the separators removed and with
       the "Type" suffix appended after the Representation Term.
  </para>
  <para>
    In light of [ELN1], this test is applied to the element's name.
  </para>
  <para>
    <emphasis>Rule problem:</emphasis> this rule's text does not accommodate
    translation of abbreviations, and the "shared" concept is poorly explained
  </para>
</xs:template>
<xsl:template name="gu:rule-CTN2" as="xsd:string*">
  <xsl:if test="gu:col(.,'ComponentType')='BBIE'">
    <xsl:variable name="name" select="gu:col(.,$gu:names)"/>
    <xsl:variable name="repTerm" select="gu:col(.,'RepresentationTerm')"/>
    <xsl:variable name="propTerm" select="gu:col(.,'PropertyTerm')"/>
    <xsl:variable name="priNoun" select="gu:col(.,'PropertyTermPrimaryNoun')"/>
    <xsl:variable name="propertyTermAbbreviated"
           select="if( $repTerm = 'Text' or
                       $priNoun = 'URI' or
                       $priNoun = 'UUID' or
                       ( $repTerm != 'Identifier' and
                         ( $propTerm = $repTerm or
                           ends-with( $propTerm, concat(' ',$repTerm) ) ) ) )
                   then if( $repTerm = 'Text' )
                        then if( $propTerm = 'Text' )
                             then 'Text'
                             else replace($propTerm,'(.)(Text)?$','$1')
                        else $propTerm
                   else if( $repTerm = 'Identifier' )
                        then if( $propTerm = 'Identifier' )
                             then 'ID'
                             else replace($propTerm,'(.)(Identifier)?$','$1ID')
                        else concat( $propTerm, $repTerm )"/>
    <xsl:variable name="expected" 
                  select="translate( concat( gu:col(.,'PropertyTermQualifier'),
                                             $propertyTermAbbreviated ),
                                     ' ','' )"/>
    <xsl:if test="$name != $expected">
      <xsl:value-of>
        <xsl:text>Mismatched element/type name "</xsl:text>
        <xsl:value-of select="$name"/>
        <xsl:text>" with property term qualifier and property term "</xsl:text>
        <xsl:value-of select="$expected"/>
        <xsl:text>" [CTN2].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>
  
<xs:template>
  <para>
       <emphasis role="bold">[CTN6]</emphasis> 5.2.2
       A UBL xsd:complexType name based on a CCTS BBIE Property and with a CCTS
       BBIE Representation Term of "Text" MUST have the word "Text" removed 
       from the end of its name.
  </para>
  <para>
    In light of [ELN1], this test is applied to the element's name.
  </para>
  <para>
    <emphasis>Rule problem:</emphasis> this rule does not accommodate a BBIE
    whose complete name is "Text", and the xsd:complexType's name ends in 
    "Type", so the "name" end of the rule is ambiguously referencing the
    element name
  </para>
</xs:template>
<xsl:template name="gu:rule-CTN6" as="xsd:string*">
  <xsl:if test="gu:col(.,'ComponentType')='BBIE'">
    <xsl:if test="gu:col(.,'RepresentationTerm')='Text' and
                  gu:col(.,$gu:names)!='Text' and
                  gu:col(.,$gu:names)/ends-with(.,'Text')">
      <xsl:value-of>
        <xsl:text>Element name for a representation term of "Text" </xsl:text>
        <xsl:text>cannot end in "Text" [CTN6].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>
  
<xs:template>
  <para>
       <emphasis role="bold">[CTN7]</emphasis> 5.2.2
       A UBL xsd:complexType name based on a CCTS BBIE Property and with a CCTS
       BBIE Representation Term of "Identifier" MUST replace "Identifier" with
       "ID" at the end of its name.
  </para>
  <para>
    In light of [ELN1], this test is applied to the element's name.
  </para>
  <para>
    <emphasis>Rule problem:</emphasis> the xsd:complexType's name already ends
    in "Type", so the "name" end of the rule is ambiguously referencing the
    element name
  </para>
  <para>
    <emphasis>Rule problem:</emphasis> the replacement of "Identifier" does
    not happen when the property term ends with "URI" or "UUID", yet the 
    representation term in both cases is "Identifier". 
  </para>
</xs:template>
<xsl:template name="gu:rule-CTN7" as="xsd:string*">
  <xsl:if test="gu:col(.,'ComponentType')='BBIE'">
    <xsl:if test="gu:col(.,'RepresentationTerm')='Identifier' and
                  gu:col(.,$gu:names)/not(ends-with(.,'ID') or 
                                          ends-with(.,'UUID') or
                                          ends-with(.,'URI'))">
      <xsl:value-of>
        <xsl:text>Element name "</xsl:text>
        <xsl:value-of select="gu:col(.,$gu:names)"/>
        <xsl:text>" for a representation term of </xsl:text>
    <xsl:text>"Identifier" must end in "ID", "UUID" or "URI" [CTN7].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>
  
<xs:template>
  <para>
       <emphasis role="bold">[CTN8]</emphasis> 5.2.2
       A UBL xsd:complexType name based on a CCTS BBIE Property MUST remove all
       duplication of words that occurs as a result of duplicate Property Terms
       and Representation Terms.
  </para>
  <para>
    In light of [ELN1], this test is applied to the element's name.
  </para>
</xs:template>
<xsl:template name="gu:rule-CTN8" as="xsd:string*">
  <xsl:if test="gu:col(.,'ComponentType')='BBIE'">
    <xsl:variable name="repTerm" 
           select="translate( gu:col(.,'RepresentationTerm')/
                              (if(.='Identifier') then 'ID' else .),' ','' )"/>
    <xsl:if test="ends-with(gu:col(.,$gu:names),concat($repTerm,$repTerm))">
      <xsl:value-of>
        <xsl:text>Element name cannot end with duplicated </xsl:text>
        <xsl:text>representation terms [CTN8].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>
  
<!--========================================================================-->
<xs:doc>
  <xs:title>[DOC] documentation rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[DOC4]</emphasis> 4.2
       The xsd:documentation element for every BBIE MUST contain a set of 
       annotations in the following order:
  </para>
<itemizedlist>
<listitem>ComponentType (mandatory): The type of component to which the object
belongs. For BBIEs this MUST be "BBIE".</listitem>
<listitem>DictionaryEntryName (mandatory): The official name of a
BBIE.</listitem>
<listitem>Version (optional): An indication of the evolution over time of the 
BBIE Entity.</listitem>
<listitem>Definition (mandatory): The meaning of a BBIE.</listitem>
<listitem>Cardinality (mandatory): Indicates whether the BBIE represents a 
not-applicable, optional, mandatory, or repetitive characteristic of 
the Aggregate Business Information Entity to which it belongs.</listitem>
<listitem>ObjectClassQualifier (optional): The qualifier for the Object 
Class.</listitem>
<listitem>ObjectClass (mandatory): The Object Class containing the 
BBIE.</listitem>
<listitem>PropertyTermQualifier (optional): A word or words which help define 
and differentiate a BBIE.</listitem>
<listitem>PropertyTerm (mandatory): Conveys the characteristic or Property of 
the Object Class.</listitem>
<listitem>RepresentationTerm (mandatory): Describes the form in which the BBIE 
is represented.</listitem>
<listitem>DataTypeQualifier (optional): A meaningful name that differentiates
the data type of the BBIE
from its underlying Core Component Type.</listitem>
<listitem>DataType (mandatory): Defines the data type used for the
BBIE.</listitem>
<listitem>AlternativeBusinessTerms (optional): Any synonymous terms under which
the BBIE is commonly known and used in the business.</listitem>
<listitem>Examples (optional): Examples of possible values for the
BBIE.</listitem>
</itemizedlist>
</xs:template>
<xsl:template name="gu:rule-DOC4" as="xsd:string*">
  <!--select all the non-empty leaf nodes of the documentation structures-->
  <xsl:variable name="fields" 
                select="type-documentation//*[not(*)][string(.)]"/>
  <xsl:if test="$ubl-specific">
    <!--checking only the mandatory fields since the optional are allowed not
        to be there-->
    <xsl:for-each select="'ComponentType','DictionaryEntryName',
                          'Definition','Cardinality','ObjectClass',
                          'PropertyTerm','RepresentationTerm','DataType'">
      <xsl:if test="not( . = $fields )">
        <xsl:value-of>
          <xsl:text>Missing the documentation directive for "</xsl:text>
          <xsl:value-of select="."/>
          <xsl:text>" [DOC4]</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[DOC5]</emphasis> 4.2
       The xsd:documentation element for every ABIE MUST contain a set of 
       annotations in the following order:
  </para>
<itemizedlist>
  <listitem>ComponentType (mandatory): The type of component to which the 
    object belongs. For ABIEs this MUST be "ABIE".</listitem>
<listitem>DictionaryEntryName (mandatory): The official name of the 
  ABIE.</listitem>
<listitem>Version (optional): An indication of the evolution over time of the
ABIE.</listitem>
<listitem>Definition (mandatory): The meaning of the ABIE.</listitem>
<listitem>ObjectClassQualifier (optional): The qualifier for the Object
Class.</listitem>
<listitem>ObjectClass (mandatory): The Object Class represented by the
ABIE.</listitem>
<listitem>AlternativeBusinessTerms (optional): Any synonymous terms under which
the ABIE is commonly known and used in the business.</listitem>
</itemizedlist>
</xs:template>
<xsl:template name="gu:rule-DOC5" as="xsd:string*">
  <!--select all the non-empty leaf nodes of the documentation structures-->
  <xsl:variable name="fields" 
                select="type-documentation//*[not(*)][string(.)]"/>
  <xsl:if test="$ubl-specific">
    <!--checking only the mandatory fields since the optional are allowed not
        to be there-->
    <xsl:for-each select="'ComponentType','DictionaryEntryName',
                          'Definition','ObjectClass'">
      <xsl:if test="not( . = $fields )">
        <xsl:value-of>
          <xsl:text>Missing the documentation directive for "</xsl:text>
          <xsl:value-of select="."/>
          <xsl:text>" [DOC5]</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[DOC6]</emphasis> 4.2
       The xsd:documentation element for every ASBIE element declaration
       MUST contain a set of annotations in the following order:
  </para>
<itemizedlist>
<listitem>ComponentType (mandatory): The type of component to which the object
belongs. For ASBIEs this MUST be "ASBIE".</listitem>
<listitem>DictionaryEntryName (mandatory): The official name of the
ASBIE.</listitem>
<listitem>Version (optional): An indication of the evolution over time of the
ASBIE.</listitem>
<listitem>Definition (mandatory): The meaning of the ASBIE.</listitem>
<listitem>Cardinality (mandatory): Indicates whether the ASBIE represents an
optional, mandatory, or repetitive assocation.</listitem>
<listitem>ObjectClass (mandatory): The Object Class containing the
ASBIE.</listitem>
<listitem>PropertyTermQualifier (optional): A word or words which help define
and identify the ASBIE.</listitem>
<listitem>PropertyTerm (mandatory): Represents the ASBIE contained by the
Association Business Information Entity.</listitem>
<listitem>AssociatedObjectClassQualifier (optional): The Associated Object 
Class Qualifiers describe the "context" of the relationship with 
another ABIE. That is, it is the role the contained ABIE plays within
its association with the containing ABIE.</listitem>
<listitem>AssociatedObjectClass (mandatory): The Object Class at the other end
of the association. It represents the ABIE contained by the ASBIE.</listitem>
</itemizedlist>
</xs:template>
<xsl:template name="gu:rule-DOC6" as="xsd:string*">
  <!--select all the non-empty leaf nodes of the documentation structures-->
  <xsl:variable name="fields" 
                select="type-documentation//*[not(*)][string(.)]"/>
  <xsl:if test="$ubl-specific">
    <!--checking only the mandatory fields since the optional are allowed not
        to be there-->
    <xsl:for-each select="'ComponentType','DictionaryEntryName',
                          'Definition','ObjectClass','PropertyTerm',
                          'AssociatedObjectClass'">
      <xsl:if test="not( . = $fields )">
        <xsl:value-of>
          <xsl:text>Missing the documentation directive for "</xsl:text>
          <xsl:value-of select="."/>
          <xsl:text>" [DOC6]</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[ELD] element declaration rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[ELD7]</emphasis> 6.3
       Empty elements MUST not be declared, except in the case of extension
       where the UBL Extensions element is used.</para>
</xs:template>
<xsl:template name="gu:rule-ELD7" as="xsd:string*">
  <xsl:if test="gu:col(.,'ComponentType')='ABIE'">
    <xsl:if test="not(gu:col(following-sibling::Row[1],'ComponentType')=
                                                            ('BBIE','ASBIE'))">
      <xsl:value-of>
        <xsl:text>An ABIE cannot be empty, in that it must have at </xsl:text>
        <xsl:text>least one BBIE or ASBIE associated with it [ELD7].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
    <emphasis role="bold">[ELD13x]</emphasis> 3.4
    The UBLVersionID element MUST be declared immediately following the UBL 
    Extensions element with xsd:minOccurs="0".
  </para>
  <para>
    <emphasis role="bold">[VER15]</emphasis> 3.6.3
    Every UBL document schema MUST declare an optional element named 
    UBLVersionID immediately following the optional UBL Extensions element.
  </para>
</xs:template>
<xsl:template name="gu:rule-ELD13x" as="xsd:string*">
  <xsl:if test="gu:col(.,'ModelName')!=$gu:thisCommonLibraryModel and
                gu:col(.,'ComponentType')='ABIE' and
                $extension='no'">
    <xsl:for-each select="following-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ComponentType')='BBIE' and
                         gu:col(.,$gu:names)='UBLVersionID' and
                         gu:col(.,'Cardinality')='0..1' )">
        <xsl:value-of>
          <xsl:text>The first BIE for a document ABIE must be </xsl:text>
          <xsl:text>an optional UBLVersionID [ELD13x][VER15].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
  <xsl:if test="gu:col(.,$gu:names)='UBLVersionID' and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:for-each select="preceding-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ModelName')!=$gu:thisCommonLibraryModel and
                         gu:col(.,'ComponentType')='ABIE' )">
        <xsl:value-of>
          <xsl:text>UBLVersionID must be the first BIE for a </xsl:text>
          <xsl:text>document ABIE [ELD13x][VER15].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
    <emphasis role="bold">[ELD14x]</emphasis> 3.4
    The CustomizationID element MUST be declared immediately following the 
    UBLVersionID element with xsd:minOccurs="0".
</para>
</xs:template>
<xsl:template name="gu:rule-ELD14x" as="xsd:string*">
  <xsl:if test="gu:col(.,$gu:names)='UBLVersionID' and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:for-each select="following-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ComponentType')='BBIE' and
                         gu:col(.,$gu:names)='CustomizationID' and
                         gu:col(.,'Cardinality')='0..1' )">
        <xsl:value-of>
          <xsl:text>The first BIE after UBLVersionID must be </xsl:text>
          <xsl:text>an optional CustomizationID [ELD14x].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
  <xsl:if test="gu:col(.,$gu:names)='CustomizationID' and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:for-each select="preceding-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ComponentType')='BBIE' and
                         gu:col(.,$gu:names)='UBLVersionID' and
                         gu:col(.,'Cardinality')='0..1' )">
        <xsl:value-of>
          <xsl:text>The first BIE prior to CustomizationID must be </xsl:text>
          <xsl:text>an optional UBLVersionID [ELD14x].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
    <emphasis role="bold">[ELD14x2]</emphasis> 3.4
    The ProfileID element MUST be declared immediately following the 
    CustomizationID element with xsd:minOccurs="0".
</para>
</xs:template>
<xsl:template name="gu:rule-ELD14x2" as="xsd:string*">
  <xsl:if test="gu:col(.,$gu:names)='CustomizationID' and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:for-each select="following-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ComponentType')='BBIE' and
                         gu:col(.,$gu:names)='ProfileID' and
                         gu:col(.,'Cardinality')='0..1' )">
        <xsl:value-of>
          <xsl:text>The first BIE after CustomizationID must be </xsl:text>
          <xsl:text>an optional ProfileID [ELD14x2].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
  <xsl:if test="gu:col(.,$gu:names)='ProfileID' and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:for-each select="preceding-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ComponentType')='BBIE' and
                         gu:col(.,$gu:names)='CustomizationID' and
                         gu:col(.,'Cardinality')='0..1' )">
        <xsl:value-of>
          <xsl:text>The first BIE prior to ProfileID must be </xsl:text>
          <xsl:text>an optional CustomizationID [ELD14x2].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
    <emphasis role="bold">[ELD14x3]</emphasis> 3.4
    The ProfileExecutionID element MUST be declared immediately following the 
    ProfileID element with xsd:minOccurs="0".
</para>
</xs:template>
<xsl:template name="gu:rule-ELD14x3" as="xsd:string*">
  <xsl:if test="gu:col(.,$gu:names)='ProfileID' and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:for-each select="following-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ComponentType')='BBIE' and
                         gu:col(.,$gu:names)='ProfileExecutionID' and
                         gu:col(.,'Cardinality')='0..1' )">
        <xsl:value-of>
          <xsl:text>The first BIE after ProfileID must be </xsl:text>
          <xsl:text>an optional ProfileExecutionID [ELD14x3].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
  <xsl:if test="gu:col(.,$gu:names)='ProfileExecutionID' and
                gu:col(.,'ComponentType')='BBIE'">
    <xsl:for-each select="preceding-sibling::Row[1]">
      <xsl:if test="not( gu:col(.,'ComponentType')='BBIE' and
                         gu:col(.,$gu:names)='ProfileID' and
                         gu:col(.,'Cardinality')='0..1' )">
        <xsl:value-of>
        <xsl:text>The first BIE prior to ProfileExecutionID must be </xsl:text>
          <xsl:text>an optional ProfileID [ELD14x3].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[ELN] element naming rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       [ELN3] 5.3.3
       A UBL global element name based on a CCTS ASBIE MUST be the CCTS ASBIE 
       Dictionary Entry Name Property Term and its qualifiers and the Object 
       Class Term and qualifiers of its associated CCTS ABIE. All CCTS 
       Dictionary Entry Name separators MUST be removed.
  </para>
</xs:template>
<xsl:template name="gu:rule-ELN3" as="xsd:string*">
  <xsl:if test="gu:col(.,'ComponentType')='ASBIE'">
    <xsl:if test="gu:col(.,'PropertyTerm')!=gu:col(.,'AssociatedObjectClass')">
      <xsl:value-of>
        <xsl:text>Mismatched ASBIE property term "</xsl:text>
        <xsl:value-of select="gu:col(.,'PropertyTerm')"/>
        <xsl:text>" and associated object class "</xsl:text>
        <xsl:value-of select="gu:col(.,'AssociatedObjectClass')"/>
        <xsl:text>" [ELN3].</xsl:text>
      </xsl:value-of>
    </xsl:if>
    <xsl:if test="gu:col(.,$gu:names) !=
                  gu:collapse( ( gu:col(.,'PropertyTermQualifier'),
                                 gu:col(.,'PropertyTerm') ) )">
      <xsl:value-of>
        <xsl:text>Mismatched element name "</xsl:text>
        <xsl:value-of select="gu:col(.,'PropertyTerm')"/>
        <xsl:text>" and property term qualifier and property term "</xsl:text>
        <xsl:value-of select="gu:col(.,'AssociatedObjectClass')"/>
        <xsl:text>" [ELN3].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[GNR] general naming rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[GNR2]</emphasis> 5.1
       UBL XML element and type names MUST be consistently derived from
       CCTS conformant Dictionary Entry Names.
  </para>
  <para>
    Note that [CTN2] already checks the element name against the property
    term and representation term, so this test only checks the Dictionary
    Entry Name.
  </para>
  <para>
    ISO/IEC 11179-1:2004 specifies the three portions of a dictionary entry
    name to be "object class", "property" and "representation".  In CCTS,
    qualifying parts of portions are suffixed with "_ ", and portions are
    separated with ". ".
  </para>
</xs:template>
<xsl:template name="gu:rule-GNR2" as="xsd:string*">
  <xsl:choose>
    <xsl:when test="gu:col(.,'ComponentType')='ABIE'">
      <xsl:if test="not( gu:col(.,$gu:names) = 
                         gu:collapse( gu:col(.,'ObjectClass' ) ) )">
        <xsl:value-of>
          <xsl:text>Mismatched element name "</xsl:text>
          <xsl:value-of select="gu:col(.,$gu:names)"/>
          <xsl:text>" with the collapsed object class "</xsl:text>
          <xsl:value-of select="gu:collapse( gu:col(.,'ObjectClass' ) )"/>
          <xsl:text>" [GNR2].</xsl:text>
        </xsl:value-of>
      </xsl:if>
      <xsl:if test="not( gu:col(.,'DictionaryEntryName') = 
                         concat( gu:col(.,'ObjectClass'), '. Details') )">
        <xsl:value-of>
          <xsl:text>Mismatched dictionary entry name "</xsl:text>
          <xsl:value-of select="gu:col(.,'DictionaryEntryName')"/>
          <xsl:text>" with the expectations of the object class "</xsl:text>
          <xsl:value-of select="gu:col(.,'ObjectClass')"/>
          <xsl:text>" [GNR2].</xsl:text>
        </xsl:value-of>
      </xsl:if>
    </xsl:when>
    <xsl:otherwise>
      <!--first check that the property term is the correct combination-->
      <xsl:if test="gu:col(.,'ComponentType')='BBIE'">
        <xsl:variable name="expectedPropertyTerm" as="xsd:string">
         <xsl:value-of>
           <xsl:for-each select="gu:col(.,'PropertyTermPossessiveNoun')">
             <xsl:value-of select="concat(.,' ')"/>
           </xsl:for-each>
           <xsl:value-of select="gu:col(.,'PropertyTermPrimaryNoun')"/>
         </xsl:value-of>
        </xsl:variable>
        <xsl:if test="not( gu:col(.,'PropertyTerm') = $expectedPropertyTerm )">
         <xsl:value-of>
           <xsl:text>Mismatched property term "</xsl:text>
           <xsl:value-of select="gu:col(.,'PropertyTerm')"/>
           <xsl:text>" with the combination of possessive noun and </xsl:text>
           <xsl:text>primary noun "</xsl:text>
           <xsl:value-of select="$expectedPropertyTerm"/>
           <xsl:text>" [GNR2].</xsl:text>
         </xsl:value-of>
        </xsl:if>
      </xsl:if>
      <!--now check that that property term is used correctly in the DEN-->
      <xsl:variable name="expectedDEN" as="xsd:string">
       <xsl:value-of>
         <xsl:for-each select="gu:col(.,'ObjectClassQualifier')">
           <xsl:value-of select="concat(.,'_ ')"/>
         </xsl:for-each>
         <xsl:value-of select="concat( gu:col(.,'ObjectClass'),'. ' )"/>
         <!--remember property in order to remove redundant representation-->
         <xsl:variable name="property" as="xsd:string">
           <xsl:value-of>
           <xsl:for-each select="gu:col(.,'PropertyTermQualifier')">
             <xsl:value-of select="concat(.,'_ ')"/>
           </xsl:for-each>
           <xsl:value-of select="gu:col(.,'PropertyTerm')"/>
           </xsl:value-of>
         </xsl:variable>
         <xsl:value-of select="$property"/>
         <!--only add the representation term if not a duplicate-->
         <xsl:for-each select="gu:col(.,'RepresentationTerm')[.!=$property]">
           <xsl:value-of select="concat('. ',.)"/>
         </xsl:for-each>
       </xsl:value-of>
      </xsl:variable>
      <xsl:if test="not( gu:col(.,'DictionaryEntryName') = $expectedDEN )">
       <xsl:value-of>
         <xsl:text>Mismatched dictionary entry name "</xsl:text>
         <xsl:value-of select="gu:col(.,'DictionaryEntryName')"/>
         <xsl:text>" with the combination of possessive noun and </xsl:text>
         <xsl:text>primary noun "</xsl:text>
         <xsl:value-of select="$expectedDEN"/>
         <xsl:text>" [GNR2].</xsl:text>
       </xsl:value-of>
      </xsl:if>
      <xsl:if test="matches( gu:col(.,'DictionaryEntryName'), '[a-z][A-Z]' )">
       <xsl:value-of>
         <xsl:text>Malformed dictionary entry name "</xsl:text>
         <xsl:value-of select="gu:col(.,'DictionaryEntryName')"/>
         <xsl:text>" because of the presence of camel case [GNR2].</xsl:text>
       </xsl:value-of>
      </xsl:if>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[GNR3]</emphasis> 5.1
       UBL XML element and type names constructed from CCTS Dictionary 
       Entry Names MUST NOT include periods, spaces, other separators, or 
       characters not allowed by XSD.
</para>
</xs:template>
<xsl:template name="gu:rule-GNR3" as="xsd:string*">
  <xsl:if test="gu:col(.,$gu:names) != gu:collapse(gu:col(.,$gu:names))">
    <xsl:value-of>
      <xsl:text>Element name "</xsl:text>
      <xsl:value-of select="gu:col(.,$gu:names)"/>
   <xsl:text>" contains punctuation reserved for use by CCTS [GNR3].</xsl:text>
    </xsl:value-of>
  </xsl:if>
  <xsl:if test="not( matches( gu:col(.,$gu:names), '^\i\c*$' ) )">
    <xsl:value-of>
      <xsl:text>Element name "</xsl:text>
      <xsl:value-of select="gu:col(.,$gu:names)"/>
      <xsl:text>" is not a valid XML name [GNR3].</xsl:text>
    </xsl:value-of>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[GNR8]</emphasis> 5.1
       The UpperCamelCase (UCC) convention MUST be used for naming elements
       and types.
</para>
</xs:template>
<xsl:template name="gu:rule-GNR8" as="xsd:string*">
  <xsl:for-each select="gu:col(.,$gu:names)[not(matches(.,$gu:UCC-abbrev))]">
    <xsl:value-of>
      <xsl:text>Element name "</xsl:text>
      <xsl:value-of select="."/>
      <xsl:text>" does not follow UpperCamelCase naming </xsl:text>
      <xsl:text>convention, accommodating the list of allowed </xsl:text>
      <xsl:text>abbreviations "</xsl:text>
      <xsl:value-of select="$gu:UCC-abbrev"/>
      <xsl:text>" [GNR8].</xsl:text>
    </xsl:value-of>
  </xsl:for-each>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[GXS] general XML schema rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[GXS2]</emphasis> 4.1
       UBL MUST provide two schemas for each transaction. One normative schema
       shall be fully annotated. One non-normative schema shall be a run-time
       schema devoid of documentation.
</para>
</xs:template>
<xsl:template name="gu:rule-GXS2" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:if test="not(//dir[ @runtime-name and @name ])">
      <xsl:value-of>
        <xsl:text>Cannot find a directory specifying names for </xsl:text>
        <xsl:text>both documented regular and undocumented runtime </xsl:text>
        <xsl:text>versions of the schemas [GXS2].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[MDC] modeling constraint rules</xs:title>
</xs:doc>

<xs:template>
  <para>
    <emphasis role="bold">[MDC1]</emphasis> 3.2.1.1.2
    UBL libraries and schemas MUST only use CCTS Core Component Types, 
    except in the case of extension, where the UBLExtensions element is used.
  </para>
  <para>
    At the model level, all BBIEs must use one of the unqualified data types
    based on CCTS Core Component Types.  At the schema level, this turns out
    to be, in fact, all based on the CCTS Core Component Types.  The
    configuration file lists all of the unqualified data types.
  </para>
</xs:template>
<xsl:template name="gu:rule-MDC1" as="xsd:string*">
  <xsl:if test="gu:col(.,'ComponentType')='BBIE'">
    <xsl:if test="not( replace( gu:col(.,'DataType'), '.*_ ', '' ) = 
                       concat( gu:col(.,'RepresentationTerm'),'. Type' ) )">
      <xsl:value-of>
        <xsl:text>Mismatched representation term "</xsl:text>
        <xsl:value-of select="gu:col(.,'RepresentationTerm')"/>
        <xsl:text>" and data type "</xsl:text>
        <xsl:value-of select="gu:col(.,'DataType')"/>
        <xsl:text>" [MDC1].</xsl:text>
      </xsl:value-of>
    </xsl:if>
    <xsl:if test="not( gu:col(.,'RepresentationTerm') = $gu:CCTS-types )">
      <xsl:value-of>
        <xsl:text>Data type "</xsl:text>
        <xsl:value-of select="gu:col(.,'RepresentationTerm')"/>
        <xsl:text>" not in set of unqualified data types (</xsl:text>
        <xsl:value-of select="$gu:CCTS-types" separator=","/>
        <xsl:text>) [MDC1].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[NMC] naming constraints rules</xs:title>
</xs:doc>

<xs:template>
  <para>
    <emphasis role="bold">[NMC1]</emphasis> 3.2.1
    Each Dictionary Entry Name MUST define one and only one fully qualified 
    path (FQP) for an element or attribute.
</para>
</xs:template>
<xsl:template name="gu:rule-NMC1" as="xsd:string*">
  <xsl:variable name="den" select="gu:col(.,'DictionaryEntryName')"/>
  <xsl:variable name="instances" 
                select="$gu:gc/key('gu:bie-by-den',$den),
                        $gu:gcOther/key('gu:bie-by-den',$den)"/>
  <xsl:if test="count($instances)>1 and . is $instances[1]">
    <xsl:value-of>
      <xsl:text>Multiple information entities (</xsl:text>
      <xsl:value-of select="count($instances)"/>
      <xsl:text>) with the same dictionary entry name are </xsl:text> 
      <xsl:text>not allowed [NMC1].</xsl:text>
    </xsl:value-of>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[NMS] namespace rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[NMS2]</emphasis> 3.5.1
       Every UBL-defined or -used major version schema set MUST have its own
       unique namespace.
  </para>
  <para>
       <emphasis role="bold">[NMS7]</emphasis> 3.7.4.1
       The UBL Common Aggregate Components schema module MUST reside in its own
       namespace.
  </para>
  <para>
       <emphasis role="bold">[NMS9]</emphasis> 3.7.4.2
       The UBL Common Basic Components schema module MUST reside in its own 
       namespace.
</para>
</xs:template>
<xsl:template name="gu:rule-NMS2" as="xsd:string*">
  <xsl:variable name="ns" select=".//file/@namespace"/>
  <xsl:for-each select="$ns">
    <xsl:if test="count($ns[.=current()])>1 and . is $ns[.=current()][1]">
      <xsl:value-of>
       <xsl:text>The namespace "</xsl:text>
       <xsl:value-of select="."/>
       <xsl:text>" is used for more than one schema fragment [NMS2].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:for-each>
  <!--run a substitution on each multiple files namespace to ensure distinct-->
  <xsl:for-each select=".//files/@namespace">
    <xsl:if test="replace( ., '%%', '%' ) (:at least assume double escape:) =
                  gu:substituteFields( '', . )">
      <xsl:value-of>
       <xsl:text>The namespace for multiple files "</xsl:text>
       <xsl:value-of select="."/>
       <xsl:text>" has no distinguishing substitution variables, </xsl:text>
       <xsl:text>thus resulting in non-unique namespace use [NMS2].</xsl:text>
      </xsl:value-of>
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[NMS8]</emphasis> 3.7.4.1
       The UBL Common Aggregate Components schema module namespace MUST be 
       represented by the namespace prefix "cac" when referenced in other 
       schemas.
</para>
</xs:template>
<xsl:template name="gu:rule-NMS8" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='CABIE'][@prefix!='cac']">
      <xsl:value-of>
        <xsl:text>Unexpected prefix for common aggregate components </xsl:text>
        <xsl:text>schema module referencing: "</xsl:text>
        <xsl:value-of select="@prefix"/>
        <xsl:text>" [NMS8].</xsl:text>
      </xsl:value-of>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[NMS10]</emphasis> 3.7.4.2
       The UBL Common Basic Components schema module namespace MUST be 
       represented by the namespace prefix "cbc" when referenced in other 
       schemas.
</para>
</xs:template>
<xsl:template name="gu:rule-NMS10" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='CBBIE'][@prefix!='cbc']">
      <xsl:value-of>
        <xsl:text>Unexpected prefix for common basic components </xsl:text>
        <xsl:text>schema module referencing: "</xsl:text>
        <xsl:value-of select="@prefix"/>
        <xsl:text>" [NMS10].</xsl:text>
      </xsl:value-of>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[NMS18]</emphasis> 3.7.4
       The CommonExtensionComponents schema module namespace MUST be 
       represented by the namespace prefix "ext" when referenced in other 
       schemas.
</para>
</xs:template>
<xsl:template name="gu:rule-NMS18" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='EXT'][@prefix!='ext']">
      <xsl:value-of>
        <xsl:text>Unexpected prefix for common extension components </xsl:text>
        <xsl:text>schema module referencing: "</xsl:text>
        <xsl:value-of select="@prefix"/>
        <xsl:text>" [NMS18].</xsl:text>
      </xsl:value-of>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       [NMS16] 3.7.4.5
       The UBL Qualified Datatypes schema module namespace MUST be represented
       by the namespace prefix "qdt" when referenced in other schemas.
</para>
</xs:template>
<xsl:template name="gu:rule-NMS16" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='QDT'][@prefix!='qdt']">
      <xsl:value-of>
        <xsl:text>Unexpected prefix for qualified data types </xsl:text>
        <xsl:text>schema module referencing: "</xsl:text>
        <xsl:value-of select="@prefix"/>
        <xsl:text>" [NMS16].</xsl:text>
      </xsl:value-of>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       [NMS16b] 3.7.4.5
       The UBL Unqualified Datatypes schema module namespace MUST be represented
       by the namespace prefix "udt" when referenced in other schemas.
</para>
</xs:template>
<xsl:template name="gu:rule-NMS16b" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='UDT'][@prefix!='udt']">
      <xsl:value-of>
        <xsl:text>Unexpected prefix for unqualified data types </xsl:text>
        <xsl:text>schema module referencing: "</xsl:text>
        <xsl:value-of select="@prefix"/>
        <xsl:text>" [NMS16b].</xsl:text>
      </xsl:value-of>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[SSM] schema structure modularity</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[SSM9]</emphasis> 3.7.4.1
       A schema module defining all UBL Common Aggregate Components MUST be 
       created.
</para>
</xs:template>
<xsl:template name="gu:rule-SSM9" as="xsd:string*">
  <xsl:if test="not(//file[@type='CABIE'])">
    <xsl:value-of>
      <xsl:text>Cannot find a file of type 'CABIE' for the common </xsl:text>
      <xsl:text>aggregate schema [SSM9].</xsl:text>
    </xsl:value-of>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[SSM10]</emphasis> 3.7.4.1
       The UBL Common Aggregate Components schema module MUST be identified 
       as CommonAggregateComponents in the document name within the schema
       header.
  </para>
  <para>
    This assumes that the file's name is copied into the file's header.
  </para>
</xs:template>
<xsl:template name="gu:rule-SSM10" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='CABIE']
                           [not(contains(@name,'CommonAggregateComponents'))]">
      <xsl:value-of>
        <xsl:text>The common aggregate schema (type='CABIE') does </xsl:text>
        <xsl:text>not include 'CommonAggregateComponents' in the </xsl:text>
        <xsl:text>name: "</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>" [SSM10].</xsl:text>
      </xsl:value-of>
      </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[SSM11]</emphasis> 3.7.4.2
       A schema module defining all UBL Common Basic Components MUST be 
       created.
</para>
</xs:template>
<xsl:template name="gu:rule-SSM11" as="xsd:string*">
  <xsl:if test="not(//file[@type='CBBIE'])">
    <xsl:value-of>
      <xsl:text>Cannot find a file of type 'CBBIE' for the common </xsl:text>
      <xsl:text>aggregate schema [SSM11].</xsl:text>
    </xsl:value-of>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[SSM12]</emphasis> 3.7.4.2
       The UBL Common Basic Components schema module MUST be identified as 
       CommonBasicComponents in the document name within the schema header.
  </para>
  <para>
    This assumes that the file's name is copied into the file's header.
  </para>
</xs:template>
<xsl:template name="gu:rule-SSM12" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='CBBIE']
                           [not(contains(@name,'CommonBasicComponents'))]">
      <xsl:value-of>
        <xsl:text>The common basic schema (type='CBBIE') does </xsl:text>
        <xsl:text>not include 'CommonBasicComponents' in the </xsl:text>
        <xsl:text>name: "</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>" [SSM12].</xsl:text>
      </xsl:value-of>
      </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[SSM19]</emphasis> 3.7.4.5
       The UBL Qualified Datatypes schema module MUST be identified as 
       QualifiedDatatypes in the document name in the schema header.
  </para>
  <para>
    Note in UBL 2.1, this is "Qualified Data Types" and "QualifiedDataTypes",
    which is not a compatibility issue since the schemas are isolated in
    each version.
  </para>
</xs:template>
<xsl:template name="gu:rule-SSM19" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='QDT']
                           [not(contains(@name,'QualifiedDataTypes'))]">
      <xsl:value-of>
        <xsl:text>The qualified data types schema (type='QDT') </xsl:text>
        <xsl:text>does not include 'QualifiedDataTypes' in the </xsl:text>
        <xsl:text>name: "</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>" [SSM19].</xsl:text>
      </xsl:value-of>
      </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[SSM19b]</emphasis> 3.7.4.5
       The UBL Unqualified Data Types schema module MUST be identified as 
       Unqualified Data Types in the document name in the schema header.
  </para>
</xs:template>
<xsl:template name="gu:rule-SSM19b" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='UDT']
                           [not(contains(@name,'UnqualifiedDataTypes'))]">
      <xsl:value-of>
        <xsl:text>The qualified data types schema (type='UDT') </xsl:text>
        <xsl:text>does not include 'UnqualifiedDataTypes' in the </xsl:text>
        <xsl:text>name: "</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>" [SSM19b].</xsl:text>
      </xsl:value-of>
      </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[SSM21]</emphasis> 3.7.4
       The UBL extension schema module MUST be identified as 
       CommonExtensionComponents in the document name within the schema header.
</para>
</xs:template>
<xsl:template name="gu:rule-SSM21" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type='EXT']
                           [not(contains(@name,'CommonExtensionComponents'))]">
      <xsl:value-of>
        <xsl:text>The qualified data types schema (type='EXT') </xsl:text>
      <xsl:text>does not include 'CommonExtensionComponents' in the </xsl:text>
        <xsl:text>name: "</xsl:text>
        <xsl:value-of select="@name"/>
        <xsl:text>" [SSM21].</xsl:text>
      </xsl:value-of>
      </xsl:for-each>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>[VER] versioning rules</xs:title>
</xs:doc>

<xs:template>
  <para>
       <emphasis role="bold">[VER2]</emphasis> 3.6.1
       Every UBL schema module major version MUST have an RFC 3121 document-id
       of the form &lt;modulename>-&lt;major>
  </para>
  <para>
       <emphasis role="bold">[VER4]</emphasis> 3.6.1
       Every minor version release of a UBL schema module MUST have a 
       document-id of the form &lt;modulename>-&lt;major>
</para>
  <para>
       <emphasis role="bold">[VER6]</emphasis> 3.6.1
       Every UBL schema module major version number MUST be a sequentially 
       assigned integer greater than zero.
</para>
  <para>
    This is assumed to be found at the end of the namespace URI.
  </para>
</xs:template>
<xsl:template name="gu:rule-VER2-VER4-VER6" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select=".//file[@type=
                                  ('CABIE','CBBIE','QDT','UDT','EXT','DABIE')]
                           [not(matches(@namespace,'-[0-9]+$'))]">
      <xsl:value-of>
        <xsl:text>The major version number as expressed at the end </xsl:text>
        <xsl:text>of the namespace URI is not numbered according </xsl:text>
        <xsl:text>to simply a major version number "</xsl:text>
        <xsl:value-of select="@namespace"/>
        <xsl:text>" for the schema of type "</xsl:text>
        <xsl:value-of select="@type"/>
        <xsl:text>" [VER2][VER4][VER6].</xsl:text>
      </xsl:value-of>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<xs:template>
  <para>
       <emphasis role="bold">[VER12]</emphasis> 3.6.2
       Every major version release of a UBL schema module MUST capture its 
       version number in the xsd:version attribute of the xsd:schema element 
       in the form &lt;major>.0
</para>
  <para>
       <emphasis role="bold">[VER14]</emphasis> 3.6.2
       Every minor version release of a UBL schema module MUST capture its 
       version information in the xsd:version attribute in the form 
       &lt;major>.&lt;non-zero>
</para>
</xs:template>
<xsl:template name="gu:rule-VER12-VER14" as="xsd:string*">
  <xsl:if test="$ubl-specific">
    <xsl:for-each select="/*/configuration[.//file/@type=
                                  ('CABIE','CBBIE','QDT','UDT','EXT','DABIE')]
                           [not(matches(@version,'^[0-9]+\.[0-9]+$'))]">
      <xsl:value-of>
        <xsl:text>The schema version attribute for the common </xsl:text>
        <xsl:text>schemas is not correctly formed "</xsl:text>
        <xsl:value-of select="@version"/>
        <xsl:text>" [VER12][VER14].</xsl:text>
      </xsl:value-of>
    </xsl:for-each>
  </xsl:if>
</xsl:template>

<!--========================================================================-->
<xs:doc>
  <xs:title>Utility templates and functions</xs:title>
</xs:doc>

<xs:function>
  <para>
    Return a row's column value based on a column reference, translating the
    property term if the representation term is "Identifier"    
  </para>
  <xs:param name="row">
    <para>The row of the genericode file.</para>
  </xs:param>
  <xs:param name="col">
    <para>
      The column reference of the value in the row.  Note that multiple
      column references are allowed, but only one of the column references
      is allowed to match.  If the row matches more than one column name
      given, this will abend in a runtime error.
    </para>
  </xs:param>
</xs:function>
<xsl:function name="gu:abbrev" as="xsd:string?">
  <xsl:param name="row" as="element(Row)"/>
  <xsl:param name="col" as="xsd:string+"/>
  <xsl:variable name="return" as="element(SimpleValue)*"
                select="$row/Value[@ColumnRef=$col]/SimpleValue"/>
  <xsl:if test="count($return) > 1">
    <xsl:message terminate="yes">
      <xsl:text>Data error: multiple genericode values in a single </xsl:text>
      <xsl:text>row for column reference</xsl:text>
      <xsl:if test="count($col)>1">s</xsl:if>: <xsl:text/>
      <xsl:value-of select="$col" separator=", "/> at <xsl:text/>
      <xsl:for-each select="$row/ancestor-or-self::*">
        <xsl:text/>/<xsl:value-of select="name(.)"/>
        <xsl:if test="self::Row">[<xsl:number/>]</xsl:if>
      </xsl:for-each>
    </xsl:message>
  </xsl:if>
  <xsl:sequence select="$return"/>
</xsl:function>
  
<xs:function>
  <para>
    Return the concatenation of parts of a name, massaged not to have
    the dictionary entry name delimeters.
  </para>
  <xs:param name="parts">
    <para>
      The parts of the returned name being massaged.
    </para>
  </xs:param>
</xs:function>
<xsl:function name="gu:collapse" as="xsd:string?">
  <xsl:param name="parts" as="xsd:string*"/>
  <xsl:sequence select="translate( string-join( $parts, '' ), '_. ', '' )"/>
</xsl:function>
  
</xsl:stylesheet>
<!--  <xsl:choose>
    <xsl:when test="gu:col(.,'ComponentType')='ABIE'">
      
    </xsl:when>
    <xsl:when test="gu:col(.,'ComponentType')='ASBIE'">
      
    </xsl:when>
    <xsl:otherwise><!-\-BBIE-\->
      
    </xsl:otherwise>
  </xsl:choose>
-->
<!--  <xsl:choose>
    <xsl:when test="gu:col(.,'ComponentType')='BBIE'">
      
    </xsl:when>
    <xsl:when test="gu:col(.,'ComponentType')='ASBIE'">
      
    </xsl:when>
    <xsl:otherwise><!-\-ABIE-\->
      
    </xsl:otherwise>
  </xsl:choose>
-->

<!--
  to do:  search "string?" and ensure "string*" is not needed
  -->